@@ -49,16 +49,27 @@ See Machnes et.al., arXiv.1011.4874
 """
 
 # import os
+import warnings
 import numpy as np
 import scipy.linalg as la
+import scipy.sparse as sp
+# QuTiP
+from qutip import Qobj
 # QuTiP logging
-import qutip.logging as logging
+import qutip.logging_utils as logging
 logger = logging.get_logger()
 # QuTiP control modules
 from qutip.control import errors
 
+def _func_deprecation(message, stacklevel=3):
+    """
+    Issue deprecation warning
+    Using stacklevel=3 will ensure message refers the function
+    calling with the deprecated parameter,
+    """
+    warnings.warn(message, DeprecationWarning, stacklevel=stacklevel)
 
-class PropagatorComputer:
+class PropagatorComputer(object):
     """
     Base for all  Propagator Computer classes
     that are used to calculate the propagators,
@@ -72,22 +83,22 @@ class PropagatorComputer:
     ----------
     log_level : integer
         level of messaging output from the logger.
-        Options are attributes of qutip.logging,
+        Options are attributes of qutip_utils.logging,
         in decreasing levels of messaging, are:
         DEBUG_INTENSE, DEBUG_VERBOSE, DEBUG, INFO, WARN, ERROR, CRITICAL
         Anything WARN or above is effectively 'quiet' execution,
         assuming everything runs as expected.
         The default NOTSET implies that the level will be taken from
         the QuTiP settings file, which by default is WARN
-        Note value should be set using set_log_level
 
     grad_exact : boolean
         indicates whether the computer class instance is capable
         of computing propagator gradients. It is used to determine
         whether to create the Dynamics prop_grad array
     """
-    def __init__(self, dynamics):
+    def __init__(self, dynamics, params=None):
         self.parent = dynamics
+        self.params = params
         self.reset()
 
     def reset(self):
@@ -95,18 +106,47 @@ class PropagatorComputer:
         reset any configuration data
         """
         self.id_text = 'PROP_COMP_BASE'
-        self.set_log_level(self.parent.log_level)
-        self.grad_exact = False
+        self.log_level = self.parent.log_level
+        self._grad_exact = False
 
-    def set_log_level(self, lvl):
+    def apply_params(self, params=None):
+        """
+        Set object attributes based on the dictionary (if any) passed in the
+        instantiation, or passed as a parameter
+        This is called during the instantiation automatically.
+        The key value pairs are the attribute name and value
+        Note: attributes are created if they do not exist already,
+        and are overwritten if they do.
+        """
+        if not params:
+            params = self.params
+
+        if isinstance(params, dict):
+            self.params = params
+            for key in params:
+                setattr(self, key, params[key])
+
+    @property
+    def log_level(self):
+        return logger.level
+
+    @log_level.setter
+    def log_level(self, lvl):
         """
         Set the log_level attribute and set the level of the logger
         that is call logger.setLevel(lvl)
         """
-        self.log_level = lvl
         logger.setLevel(lvl)
 
+    def grad_exact(self):
+        return self._grad_exact
+
     def compute_propagator(self, k):
+        _func_deprecation("'compute_propagator' has been replaced "
+                        "by '_compute_propagator'")
+        return self._compute_propagator(k)
+                               
+    def _compute_propagator(self, k):
         """
         calculate the progator between X(k) and X(k+1)
         Uses matrix expm of the dyn_gen at that point (in time)
@@ -114,10 +154,20 @@ class PropagatorComputer:
         i.e. drift and ctrls combined
         Return the propagator
         """
-        raise errors.UsageError("Not implemented in the baseclass."
-                                " Choose a subclass")
+        dyn = self.parent
+        dgt = dyn._get_phased_dyn_gen(k)*dyn.tau[k]
+        if dyn.oper_dtype == Qobj:
+            prop = dgt.expm()
+        else:
+            prop = la.expm(dgt)
+        return prop
 
     def compute_diff_prop(self, k, j, epsilon):
+        _func_deprecation("'compute_diff_prop' has been replaced "
+                        "by '_compute_diff_prop'")
+        return self._compute_diff_prop( k, j, epsilon)
+
+    def _compute_diff_prop(self, k, j, epsilon):
         """
         Calculate the propagator from the current point to a trial point
         a distance 'epsilon' (change in amplitude)
@@ -128,6 +178,11 @@ class PropagatorComputer:
                                 " Choose a subclass")
 
     def compute_prop_grad(self, k, j, compute_prop=True):
+        _func_deprecation("'compute_prop_grad' has been replaced "
+                        "by '_compute_prop_grad'")
+        return self._compute_prop_grad(self, k, j, compute_prop=compute_prop)
+
+    def _compute_prop_grad(self, k, j, compute_prop=True):
         """
         Calculate the gradient of propagator wrt the control amplitude
         in the timeslot.
@@ -142,9 +197,6 @@ class PropCompApproxGrad(PropagatorComputer):
     by expm of the dynamics generator, i.e. when gradients will be calculated
     using approximate methods.
     """
-    def __init__(self, dynamics):
-        self.parent = dynamics
-        self.reset()
 
     def reset(self):
         """
@@ -153,21 +205,9 @@ class PropCompApproxGrad(PropagatorComputer):
         PropagatorComputer.reset(self)
         self.id_text = 'APPROX'
         self.grad_exact = False
+        self.apply_params()
 
-    def compute_propagator(self, k):
-        """
-        calculate the progator between X(k) and X(k+1)
-        Uses matrix expm of the dyn_gen at that point (in time)
-        Assumes that the dyn_gen have been been calculated,
-        i.e. drift and ctrls combined
-        Return the propagator
-        """
-        dyn = self.parent
-        dgt = dyn.get_dyn_gen(k)*dyn.tau[k]
-        prop = la.expm(dgt)
-        return prop
-
-    def compute_diff_prop(self, k, j, epsilon):
+    def _compute_diff_prop(self, k, j, epsilon):
         """
         Calculate the propagator from the current point to a trial point
         a distance 'epsilon' (change in amplitude)
@@ -175,9 +215,14 @@ class PropCompApproxGrad(PropagatorComputer):
         Returns the propagator
         """
         dyn = self.parent
-        dgt_eps = np.asarray(dyn.get_dyn_gen(k) +
-                             epsilon*dyn.get_ctrl_dyn_gen(j))
-        prop_eps = la.expm(dgt_eps*dyn.tau[k])
+        dgt_eps = (dyn._get_phased_dyn_gen(k) +
+                epsilon*dyn._get_phased_ctrl_dyn_gen(k, j))*dyn.tau[k]
+
+        if dyn.oper_dtype == Qobj:
+            prop_eps = dgt_eps.expm()
+        else:
+            prop_eps = la.expm(dgt_eps)
+
         return prop_eps
 
 
@@ -193,22 +238,29 @@ class PropCompDiag(PropagatorComputer):
         PropagatorComputer.reset(self)
         self.id_text = 'DIAG'
         self.grad_exact = True
+        self.apply_params()
 
-    def compute_propagator(self, k):
+    def _compute_propagator(self, k):
         """
         Calculates the exponentiation of the dynamics generator (H)
         As part of the calc the the eigen decomposition is required, which
         is reused in the propagator gradient calculation
         """
         dyn = self.parent
-        dyn.ensure_decomp_curr(k)
+        dyn._ensure_decomp_curr(k)
+
+        if dyn.oper_dtype == Qobj:
+
+            prop = (dyn._dyn_gen_eigenvectors[k]*dyn._prop_eigen[k]*
+                                dyn._get_dyn_gen_eigenvectors_adj(k))
+        else:
+            prop = dyn._dyn_gen_eigenvectors[k].dot(
+                                    dyn._prop_eigen[k]).dot(
+                                dyn._get_dyn_gen_eigenvectors_adj(k))
 
-        eig_vec = dyn.dyn_gen_eigenvectors[k]
-        prop_eig_diag = np.diagflat(dyn.prop_eigen[k])
-        prop = eig_vec.dot(prop_eig_diag).dot(eig_vec.conj().T)
         return prop
 
-    def compute_prop_grad(self, k, j, compute_prop=True):
+    def _compute_prop_grad(self, k, j, compute_prop=True):
         """
         Calculate the gradient of propagator wrt the control amplitude
         in the timeslot.
@@ -217,25 +269,32 @@ class PropCompDiag(PropagatorComputer):
             [prop], prop_grad
         """
         dyn = self.parent
-        dyn.ensure_decomp_curr(k)
+        dyn._ensure_decomp_curr(k)
 
         if compute_prop:
-            prop = self.compute_propagator(k)
+            prop = self._compute_propagator(k)
 
-        eig_vec = dyn.dyn_gen_eigenvectors[k]
-        eig_vec_adj = eig_vec.conj().T
-
-        # compute ctrl dyn gen in diagonalised basis
-        # i.e. the basis of the full dyn gen for this timeslot
-        dg_diag = eig_vec_adj.dot(dyn.get_ctrl_dyn_gen(j)).dot(eig_vec)
-
-        # multiply by factor matrix
-        factors = dyn.dyn_gen_factormatrix[k]
-        # note have to use multiply method as .dot returns matrix
-        # and hence * implies inner product i.e. dot
-        dg_diag_fact = np.multiply(dg_diag, factors)
-        # Return to canonical basis
-        prop_grad = eig_vec.dot(dg_diag_fact).dot(eig_vec_adj)
+        if dyn.oper_dtype == Qobj:
+            # put control dyn_gen in combined dg diagonal basis
+            cdg =  (dyn._get_dyn_gen_eigenvectors_adj(k)*
+                        dyn._get_phased_ctrl_dyn_gen(k, j)*
+                        dyn._dyn_gen_eigenvectors[k])
+            # multiply (elementwise) by timeslice and factor matrix
+            cdg = Qobj(np.multiply(cdg.full()*dyn.tau[k],
+                        dyn._dyn_gen_factormatrix[k]), dims=dyn.dyn_dims)
+            # Return to canonical basis
+            prop_grad = (dyn._dyn_gen_eigenvectors[k]*cdg*
+                        dyn._get_dyn_gen_eigenvectors_adj(k))
+        else:
+            # put control dyn_gen in combined dg diagonal basis
+            cdg =  dyn._get_dyn_gen_eigenvectors_adj(k).dot(
+                        dyn._get_phased_ctrl_dyn_gen(k, j)).dot(
+                        dyn._dyn_gen_eigenvectors[k])
+            # multiply (elementwise) by timeslice and factor matrix
+            cdg = np.multiply(cdg*dyn.tau[k], dyn._dyn_gen_factormatrix[k])
+            # Return to canonical basis
+            prop_grad = dyn._dyn_gen_eigenvectors[k].dot(cdg).dot(
+                        dyn._get_dyn_gen_eigenvectors_adj(k))
 
         if compute_prop:
             return prop, prop_grad
@@ -262,8 +321,9 @@ class PropCompAugMat(PropagatorComputer):
         PropagatorComputer.reset(self)
         self.id_text = 'AUG_MAT'
         self.grad_exact = True
+        self.apply_params()
 
-    def get_aug_mat(self, k, j):
+    def _get_aug_mat(self, k, j):
         """
         Generate the matrix [[A, E], [0, A]] where
             A is the overall dynamics generator
@@ -272,16 +332,26 @@ class PropCompAugMat(PropagatorComputer):
         returns this augmented matrix
         """
         dyn = self.parent
-        A = dyn.get_dyn_gen(k)*dyn.tau[k]
-        E = dyn.get_ctrl_dyn_gen(j)*dyn.tau[k]
-
-        l = np.concatenate((A, np.zeros(A.shape)))
-        r = np.concatenate((E, A))
-        aug = np.concatenate((l, r), 1)
+        dg = dyn._get_phased_dyn_gen(k)
 
+        if dyn.oper_dtype == Qobj:
+            A = dg.data*dyn.tau[k]
+            E = dyn._get_phased_ctrl_dyn_gen(k, j).data*dyn.tau[k]
+            Z = sp.csr_matrix(dg.data.shape)
+            aug = Qobj(sp.vstack([sp.hstack([A, E]), sp.hstack([Z, A])]))
+        elif dyn.oper_dtype == np.ndarray:
+            A = dg*dyn.tau[k]
+            E = dyn._get_phased_ctrl_dyn_gen(k, j)*dyn.tau[k]
+            Z = np.zeros(dg.shape)
+            aug = np.vstack([np.hstack([A, E]), np.hstack([Z, A])])
+        else:
+            A = dg*dyn.tau[k]
+            E = dyn._get_phased_ctrl_dyn_gen(k, j)*dyn.tau[k]
+            Z = dg*0.0
+            aug = sp.vstack([sp.hstack([A, E]), sp.hstack([Z, A])])
         return aug
 
-    def compute_prop_grad(self, k, j, compute_prop=True):
+    def _compute_prop_grad(self, k, j, compute_prop=True):
         """
         Calculate the gradient of propagator wrt the control amplitude
         in the timeslot using the exponentiation of the the augmented
@@ -292,12 +362,24 @@ class PropCompAugMat(PropagatorComputer):
             [prop], prop_grad
         """
         dyn = self.parent
-        dyn_gen_shp = dyn.get_dyn_gen(k).shape
-        aug = self.get_aug_mat(k, j)
-        aug_exp = la.expm(aug)
-        prop_grad = aug_exp[:dyn_gen_shp[0], dyn_gen_shp[1]:]
+        dg = dyn._get_phased_dyn_gen(k)
+        aug = self._get_aug_mat(k, j)
+
+
+        if dyn.oper_dtype == Qobj:
+            aug_exp = aug.expm()
+            prop_grad = Qobj(aug_exp.data[:dg.shape[0], dg.shape[1]:],
+                         dims=dyn.dyn_dims)
+            if compute_prop:
+                prop = Qobj(aug_exp.data[:dg.shape[0], :dg.shape[1]],
+                            dims=dyn.dyn_dims)
+        else:
+            aug_exp = la.expm(aug)
+            prop_grad = aug_exp[:dg.shape[0], dg.shape[1]:]
+            if compute_prop:
+                prop = aug_exp[:dg.shape[0], :dg.shape[1]]
+
         if compute_prop:
-            prop = aug_exp[:dyn_gen_shp[0], :dyn_gen_shp[1]]
             return prop, prop_grad
         else:
             return prop_grad
@@ -315,8 +397,9 @@ class PropCompFrechet(PropagatorComputer):
         PropagatorComputer.reset(self)
         self.id_text = 'FRECHET'
         self.grad_exact = True
+        self.apply_params()
 
-    def compute_prop_grad(self, k, j, compute_prop=True):
+    def _compute_prop_grad(self, k, j, compute_prop=True):
         """
         Calculate the gradient of propagator wrt the control amplitude
         in the timeslot using the expm_frechet method
@@ -326,12 +409,41 @@ class PropCompFrechet(PropagatorComputer):
             [prop], prop_grad
         """
         dyn = self.parent
-        A = dyn.get_dyn_gen(k)*dyn.tau[k]
-        E = dyn.get_ctrl_dyn_gen(j)*dyn.tau[k]
+
+        if dyn.oper_dtype == Qobj:
+            A = dyn._get_phased_dyn_gen(k).full()*dyn.tau[k]
+            E = dyn._get_phased_ctrl_dyn_gen(k, j).full()*dyn.tau[k]
+            if compute_prop:
+                prop_dense, prop_grad_dense = la.expm_frechet(A, E)
+                prop = Qobj(prop_dense, dims=dyn.dyn_dims)
+                prop_grad = Qobj(prop_grad_dense,
+                                            dims=dyn.dyn_dims)
+            else:
+                prop_grad_dense = la.expm_frechet(A, E, compute_expm=False)
+                prop_grad = Qobj(prop_grad_dense,
+                                            dims=dyn.dyn_dims)
+        elif dyn.oper_dtype == np.ndarray:
+            A = dyn._get_phased_dyn_gen(k)*dyn.tau[k]
+            E = dyn._get_phased_ctrl_dyn_gen(k, j)*dyn.tau[k]
+            if compute_prop:
+                prop, prop_grad = la.expm_frechet(A, E)
+            else:
+                prop_grad = la.expm_frechet(A, E,
+                                                    compute_expm=False)
+        else:
+            # Assuming some sparse matrix
+            spcls = dyn._dyn_gen[k].__class__
+            A = (dyn._get_phased_dyn_gen(k)*dyn.tau[k]).toarray()
+            E = (dyn._get_phased_ctrl_dyn_gen(k, j)*dyn.tau[k]).toarray()
+            if compute_prop:
+                prop_dense, prop_grad_dense = la.expm_frechet(A, E)
+                prop = spcls(prop_dense)
+                prop_grad = spcls(prop_grad_dense)
+            else:
+                prop_grad_dense = la.expm_frechet(A, E, compute_expm=False)
+                prop_grad = spcls(prop_grad_dense)
 
         if compute_prop:
-            prop, propGrad = la.expm_frechet(A, E)
-            return prop, propGrad
+            return prop, prop_grad
         else:
-            propGrad = la.expm_frechet(A, E, compute_expm=False)
-            return propGrad
+            return prop_grad
