@@ -66,16 +66,27 @@ See Machnes et.al., arXiv.1011.4874
 """
 
 import os
+import warnings
 import numpy as np
 import timeit
+# QuTiP
+from qutip import Qobj
 # QuTiP control modules
 import qutip.control.errors as errors
+import qutip.control.dump as qtrldump
 # QuTiP logging
-import qutip.logging as logging
+import qutip.logging_utils as logging
 logger = logging.get_logger()
 
+def _func_deprecation(message, stacklevel=3):
+    """
+    Issue deprecation warning
+    Using stacklevel=3 will ensure message refers the function
+    calling with the deprecated parameter,
+    """
+    warnings.warn(message, DeprecationWarning, stacklevel=stacklevel)
 
-class TimeslotComputer:
+class TimeslotComputer(object):
     """
     Base class for all Timeslot Computers
     Note: this must be instantiated with a Dynamics object, that is the
@@ -85,28 +96,49 @@ class TimeslotComputer:
     ----------
     log_level : integer
         level of messaging output from the logger.
-        Options are attributes of qutip.logging,
+        Options are attributes of qutip.logging_utils,
         in decreasing levels of messaging, are:
         DEBUG_INTENSE, DEBUG_VERBOSE, DEBUG, INFO, WARN, ERROR, CRITICAL
         Anything WARN or above is effectively 'quiet' execution,
         assuming everything runs as expected.
         The default NOTSET implies that the level will be taken from
         the QuTiP settings file, which by default is WARN
-        Note value should be set using set_log_level
-
-
+        
+    evo_comp_summary : EvoCompSummary
+        A summary of the most recent evolution computation
+        Used in the stats and dump
+        Will be set to None if neither stats or dump are set
     """
-    def __init__(self, dynamics):
+    def __init__(self, dynamics, params=None):
+        from qutip.control.dynamics import Dynamics
+        if not isinstance(dynamics, Dynamics):
+            raise TypeError("Must instantiate with {} type".format(
+                                        Dynamics))
         self.parent = dynamics
+        self.params = params
         self.reset()
 
     def reset(self):
-        self.set_log_level(self.parent.log_level)
+        self.log_level = self.parent.log_level
         self.id_text = 'TS_COMP_BASE'
-        self._fwd_evo_tofh = 0
-        self._owd_evo_tofh = 0
-        self._prop_tofh = 0
-        self._prop_grad_tofh = 0
+        self.evo_comp_summary = None
+
+    def apply_params(self, params=None):
+        """
+        Set object attributes based on the dictionary (if any) passed in the
+        instantiation, or passed as a parameter
+        This is called during the instantiation automatically.
+        The key value pairs are the attribute name and value
+        Note: attributes are created if they do not exist already,
+        and are overwritten if they do.
+        """
+        if not params:
+            params = self.params
+
+        if isinstance(params, dict):
+            self.params = params
+            for key in params:
+                setattr(self, key, params[key])
 
     def flag_all_calc_now(self):
         pass
@@ -114,14 +146,40 @@ class TimeslotComputer:
     def init_comp(self):
         pass
 
-    def set_log_level(self, lvl):
+    @property
+    def log_level(self):
+        return logger.level
+
+    @log_level.setter
+    def log_level(self, lvl):
         """
         Set the log_level attribute and set the level of the logger
         that is call logger.setLevel(lvl)
         """
-        self.log_level = lvl
         logger.setLevel(lvl)
-
+        
+    def dump_current(self):
+        """Store a copy of the current time evolution"""
+        dyn = self.parent
+        dump = dyn.dump
+        if not isinstance(dump, qtrldump.DynamicsDump):
+            raise RuntimeError("Cannot dump current evolution, "
+                "as dynamics.dump is not set")
+        
+        anything_dumped = False
+        item_idx = None
+        if dump.dump_any:
+            dump_item = dump.add_evo_dump()
+            item_idx = dump_item.idx
+            anything_dumped = True
+        
+        if dump.dump_summary:
+            dump.add_evo_comp_summary(dump_item_idx=item_idx)
+            anything_dumped = True
+                
+        if not anything_dumped:
+            logger.warning("Dump set, but nothing dumped, check dump config")
+            
 
 class TSlotCompUpdateAll(TimeslotComputer):
     """
@@ -132,6 +190,7 @@ class TSlotCompUpdateAll(TimeslotComputer):
     def reset(self):
         TimeslotComputer.reset(self)
         self.id_text = 'ALL'
+        self.apply_params()
 
     def compare_amps(self, new_amps):
         """
@@ -141,13 +200,20 @@ class TSlotCompUpdateAll(TimeslotComputer):
         """
         changed = False
         dyn = self.parent
+        
+        if (dyn.stats or dyn.dump):
+            if self.evo_comp_summary:
+                self.evo_comp_summary.reset()
+            else:
+                self.evo_comp_summary = EvoCompSummary()
+        ecs = self.evo_comp_summary
+        
         if dyn.ctrl_amps is None:
             # Flag fidelity and gradients as needing recalculation
             changed = True
-            if dyn.stats is not None:
-                dyn.stats.num_ctrl_amp_updates = 1
-                dyn.stats.num_ctrl_amp_changes = len(new_amps.flat)
-                dyn.stats.num_timeslot_changes = new_amps.shape[0]
+            if ecs:
+                ecs.num_amps_changed = len(new_amps.flat)
+                ecs.num_timeslots_changed = new_amps.shape[0]
         else:
             # create boolean array with same shape as ctrl_amps
             # True where value in new_amps differs, otherwise false
@@ -158,17 +224,21 @@ class TSlotCompUpdateAll(TimeslotComputer):
                 if self.log_level <= logging.DEBUG:
                     logger.debug("{} amplitudes changed".format(
                         changed_amps.sum()))
+                
+                if ecs:
+                    ecs.num_amps_changed = changed_amps.sum()
+                    ecs.num_timeslots_changed = np.any(changed_amps, 1).sum()
 
-                # *** update stats ***
-                if dyn.stats is not None:
-                    dyn.stats.num_ctrl_amp_updates += 1
-                    dyn.stats.num_ctrl_amp_changes += changed_amps.sum()
-                    changed_ts_mask = np.any(changed_amps, 1)
-                    dyn.stats.num_timeslot_changes += changed_ts_mask.sum()
             else:
                 if self.log_level <= logging.DEBUG:
                     logger.debug("No amplitudes changed")
 
+        # *** update stats ***
+        if dyn.stats:
+            dyn.stats.num_ctrl_amp_updates += bool(ecs.num_amps_changed)
+            dyn.stats.num_ctrl_amp_changes += ecs.num_amps_changed
+            dyn.stats.num_timeslot_changes += ecs.num_timeslots_changed
+            
         if changed:
             dyn.ctrl_amps = new_amps
             dyn.flag_system_changed()
@@ -186,7 +256,20 @@ class TSlotCompUpdateAll(TimeslotComputer):
         dyn = self.parent
         prop_comp = dyn.prop_computer
         n_ts = dyn.num_tslots
-        n_ctrls = dyn.get_num_ctrls()
+        n_ctrls = dyn.num_ctrls
+
+        # Clear the public lists
+        # These are only set if (external) users access them
+        dyn._dyn_gen_qobj = None
+        dyn._prop_qobj = None
+        dyn._prop_grad_qobj = None
+        dyn._fwd_evo_qobj = None
+        dyn._onwd_evo_qobj = None
+        dyn._onto_evo_qobj = None
+        
+        if (dyn.stats or dyn.dump) and not self.evo_comp_summary:
+            self.evo_comp_summary = EvoCompSummary()
+        ecs = self.evo_comp_summary
 
         if dyn.stats is not None:
             dyn.stats.num_tslot_recompute += 1
@@ -196,141 +279,90 @@ class TSlotCompUpdateAll(TimeslotComputer):
                                dyn.stats.num_tslot_recompute))
 
         # calculate the Hamiltonians
-        time_start = timeit.default_timer()
+        if ecs: time_start = timeit.default_timer()
         for k in range(n_ts):
-            dyn.dyn_gen[k] = dyn.combine_dyn_gen(k)
-            if dyn.decomp_curr is not None:
-                dyn.decomp_curr[k] = False
-        if dyn.stats is not None:
-            dyn.stats.wall_time_dyn_gen_compute += \
+            dyn._combine_dyn_gen(k)
+            if dyn._decomp_curr is not None:
+                dyn._decomp_curr[k] = False
+                
+        if ecs:
+            ecs.wall_time_dyn_gen_compute = \
                 timeit.default_timer() - time_start
 
-        if (dyn.config.test_out_prop or
-                dyn.config.test_out_prop_grad or
-                dyn.config.test_out_evo):
-            f_ext = "_{}_{}_{}_call{}{}".format(dyn.id_text,
-                                                dyn.prop_computer.id_text,
-                                                dyn.fid_computer.id_text,
-                                                dyn.stats.num_tslot_recompute,
-                                                dyn.config.test_out_f_ext)
-
         # calculate the propagators and the propagotor gradients
-        if dyn.config.test_out_prop:
-            fname = "prop" + f_ext
-            fpath = os.path.join(dyn.config.test_out_dir, fname)
-            self._prop_tofh = open(fpath, 'w')
-        else:
-            self._prop_tofh = 0
-
-        if dyn.config.test_out_prop_grad:
-            fname = "prop_grad" + f_ext
-            fpath = os.path.join(dyn.config.test_out_dir, fname)
-            self._prop_grad_tofh = open(fpath, 'w')
-        else:
-            self._prop_grad_tofh = 0
-
-        time_start = timeit.default_timer()
+        if ecs: time_start = timeit.default_timer()
         for k in range(n_ts):
-            if prop_comp.grad_exact:
+            if prop_comp.grad_exact and dyn.cache_prop_grad:
                 for j in range(n_ctrls):
                     if j == 0:
-                        prop, prop_grad = prop_comp.compute_prop_grad(k, j)
-                        dyn.prop[k] = prop
-                        dyn.prop_grad[k, j] = prop_grad
+                        dyn._prop[k], dyn._prop_grad[k, j] = \
+                                    prop_comp._compute_prop_grad(k, j)
                         if self.log_level <= logging.DEBUG_INTENSE:
                             logger.log(logging.DEBUG_INTENSE,
                                        "propagator {}:\n{:10.3g}".format(
-                                           k, prop))
-                        if self._prop_tofh != 0:
-                            self._prop_tofh.write(
-                                "propagator k={}\n".format(k))
-                            np.savetxt(self._prop_tofh, prop, fmt='%10.3g')
-
+                                           k, self._prop[k]))
                     else:
-                        prop_grad = prop_comp.compute_prop_grad(
-                            k, j, compute_prop=False)
-                        dyn.prop_grad[k, j] = prop_grad
-
-                    if self._prop_grad_tofh != 0:
-                        self._prop_grad_tofh.write(
-                            "prop grad k={}, j={}\n".format(k, j))
-                        np.savetxt(self._prop_grad_tofh, prop_grad,
-                                   fmt='%10.3g')
+                        dyn._prop_grad[k, j] = \
+                            prop_comp._compute_prop_grad(k, j, 
+                                                         compute_prop=False)
             else:
-                dyn.prop[k] = prop_comp.compute_propagator(k)
-                if self._prop_tofh != 0:
-                    self._prop_tofh.write(
-                        "propagator k={}\n".format(k))
-                    np.savetxt(self._prop_tofh, prop, fmt='%10.3g')
-
-        if dyn.stats is not None:
-            dyn.stats.wall_time_prop_compute += \
+                dyn._prop[k] = prop_comp._compute_propagator(k)
+        
+        if ecs:
+            ecs.wall_time_prop_compute = \
                 timeit.default_timer() - time_start
 
-        if self._prop_tofh != 0:
-            self._prop_tofh.close()
-            self._prop_tofh = 0
-        if self._prop_grad_tofh != 0:
-            self._prop_grad_tofh.close()
-            self._prop_grad_tofh = 0
-
+        if ecs: time_start = timeit.default_timer()
         # compute the forward propagation
-        if dyn.config.test_out_evo:
-            fname = "fwd_evo" + f_ext
-            fpath = os.path.join(dyn.config.test_out_dir, fname)
-            self._fwd_evo_tofh = open(fpath, 'w')
-
-            fname = "owd_evo" + f_ext
-            fpath = os.path.join(dyn.config.test_out_dir, fname)
-            self._owd_evo_tofh = open(fpath, 'w')
-        else:
-            self._fwd_evo_tofh = 0
-            self._owd_evo_tofh = 0
-
-        time_start = timeit.default_timer()
-        R = range(1, n_ts+1)
+        R = range(n_ts)
         for k in R:
-            dyn.evo_init2t[k] = dyn.prop[k-1].dot(dyn.evo_init2t[k-1])
-            if self._fwd_evo_tofh != 0:
-                self._fwd_evo_tofh.write("Evo start to k={}\n".format(k))
-                np.savetxt(self._fwd_evo_tofh, dyn.evo_init2t[k],
-                           fmt='%10.3g')
+            if dyn.oper_dtype == Qobj:
+                dyn._fwd_evo[k+1] = dyn._prop[k]*dyn._fwd_evo[k]
+            else:
+                dyn._fwd_evo[k+1] = dyn._prop[k].dot(dyn._fwd_evo[k])
 
-        if dyn.stats is not None:
-            dyn.stats.wall_time_fwd_prop_compute += \
-                timeit.default_timer() - time_start
-
-        time_start = timeit.default_timer()
+        if ecs:
+            ecs.wall_time_fwd_prop_compute = \
+                        timeit.default_timer() - time_start
+            time_start = timeit.default_timer()
         # compute the onward propagation
-        if dyn.fid_computer.uses_evo_t2end:
-            dyn.evo_t2end[n_ts - 1] = dyn.prop[n_ts - 1]
+        if dyn.fid_computer.uses_onwd_evo:
+            dyn._onwd_evo[n_ts-1] = dyn._prop[n_ts-1]
             R = range(n_ts-2, -1, -1)
             for k in R:
-                dyn.evo_t2end[k] = dyn.evo_t2end[k+1].dot(dyn.prop[k])
-                if self._fwd_evo_tofh != 0:
-                    self._owd_evo_tofh.write("Evo k={} to end:\n".format(k))
-                    np.savetxt(self._owd_evo_tofh, dyn.evo_t2end[k],
-                               fmt='%14.6g')
+                if dyn.oper_dtype == Qobj:
+                    dyn._onwd_evo[k] = dyn._onwd_evo[k+1]*dyn._prop[k]
+                else:
+                    dyn._onwd_evo[k] = dyn._onwd_evo[k+1].dot(dyn._prop[k])
 
-        if dyn.fid_computer.uses_evo_t2targ:
+        if dyn.fid_computer.uses_onto_evo:
+            #R = range(n_ts-1, -1, -1)
             R = range(n_ts-1, -1, -1)
             for k in R:
-                dyn.evo_t2targ[k] = dyn.evo_t2targ[k+1].dot(dyn.prop[k])
-                if self._fwd_evo_tofh != 0:
-                    self._owd_evo_tofh.write("Evo k={} to targ:\n".format(k))
-                    np.savetxt(self._owd_evo_tofh, dyn.evo_t2targ[k],
-                               fmt='%14.6g')
+                if dyn.oper_dtype == Qobj:
+                    dyn._onto_evo[k] = dyn._onto_evo[k+1]*dyn._prop[k]
+                else:
+                    dyn._onto_evo[k] = dyn._onto_evo[k+1].dot(dyn._prop[k])
 
-        if dyn.stats is not None:
+        if ecs:
+            ecs.wall_time_onwd_prop_compute = \
+                            timeit.default_timer() - time_start
+            
+        if dyn.stats:
+            dyn.stats.wall_time_dyn_gen_compute += \
+                                    ecs.wall_time_dyn_gen_compute
+            dyn.stats.wall_time_prop_compute += \
+                                    ecs.wall_time_prop_compute
+            dyn.stats.wall_time_fwd_prop_compute += \
+                                    ecs.wall_time_fwd_prop_compute
             dyn.stats.wall_time_onwd_prop_compute += \
-                timeit.default_timer() - time_start
-
-        if self._fwd_evo_tofh != 0:
-            self._fwd_evo_tofh.close()
-            self._fwd_evo_tofh = 0
-        if self._owd_evo_tofh != 0:
-            self._owd_evo_tofh.close()
-            self._owd_evo_tofh = 0
+                                    ecs.wall_time_onwd_prop_compute
+                
+        if dyn.unitarity_check_level:
+            dyn.check_unitarity()
+            
+        if dyn.dump:
+            self.dump_current()
 
     def get_timeslot_for_fidelity_calc(self):
         """
@@ -338,7 +370,17 @@ class TSlotCompUpdateAll(TimeslotComputer):
         value.
         This (default) method simply returns the last timeslot
         """
-        return self.parent.num_tslots - 1
+        _func_deprecation("'get_timeslot_for_fidelity_calc' is deprecated. "
+                        "Use '_get_timeslot_for_fidelity_calc'")
+        return self._get_timeslot_for_fidelity_calc
+
+    def _get_timeslot_for_fidelity_calc(self):
+        """
+        Returns the timeslot index that will be used calculate current fidelity
+        value.
+        This (default) method simply returns the last timeslot
+        """
+        return self.parent.num_tslots
 
 
 class TSlotCompDynUpdate(TimeslotComputer):
@@ -366,6 +408,7 @@ class TSlotCompDynUpdate(TimeslotComputer):
         self.evo_t2targ_calc_now = None
         TimeslotComputer.reset(self)
         self.id_text = 'DYNAMIC'
+        self.apply_params()
 
     def init_comp(self):
         """
@@ -522,7 +565,7 @@ class TSlotCompDynUpdate(TimeslotComputer):
                 if prop_recomp_now[k]:
                     # calculate exp(H) and other per H computations needed for
                     # the gradient function
-                    dyn.prop[k] = dyn.compute_propagator(k)
+                    dyn.prop[k] = dyn._compute_propagator(k)
                     self.prop_recalc[k] = False
             if dyn.stats is not None:
                 dyn.stats.num_prop_computes += prop_recomp_now.sum()
@@ -598,3 +641,94 @@ class TSlotCompDynUpdate(TimeslotComputer):
         self.evo_init2t_calc_now[kUse] = True
         self.evo_t2targ_calc_now[kUse] = True
         return kUse
+
+class EvoCompSummary(qtrldump.DumpSummaryItem):
+    """
+    A summary of the most recent time evolution computation
+    Used in stats calculations and for data dumping
+    
+    Attributes
+    ----------
+    evo_dump_idx : int
+        Index of the linked :class:`dump.EvoCompDumpItem`
+        None if no linked item
+        
+    iter_num : int
+        Iteration number of the pulse optimisation
+        None if evolution compute outside of a pulse optimisation
+        
+    fid_func_call_num : int
+        Fidelity function call number of the pulse optimisation
+        None if evolution compute outside of a pulse optimisation
+        
+    grad_func_call_num : int
+        Gradient function call number of the pulse optimisation
+        None if evolution compute outside of a pulse optimisation
+        
+    num_amps_changed : int
+        Number of control timeslot amplitudes changed since previous
+        evolution calculation
+        
+    num_timeslots_changed : int
+        Number of timeslots in which any amplitudes changed since previous
+        evolution calculation
+        
+    wall_time_dyn_gen_compute : float
+        Time spent computing dynamics generators
+        (in seconds of elapsed time)
+        
+    wall_time_prop_compute : float
+        Time spent computing propagators (including and propagator gradients)
+        (in seconds of elapsed time)
+        
+    wall_time_fwd_prop_compute : float
+        Time spent computing the forward evolution of the system
+        see :property:`dynamics.fwd_evo`  
+        (in seconds of elapsed time)
+        
+    wall_time_onwd_prop_compute : float
+        Time spent computing the 'backward' evolution of the system
+        see :property:`dynamics.onwd_evo` and :property:`dynamics.onto_evo`
+        (in seconds of elapsed time)
+    """
+    
+    min_col_width = 11
+    summary_property_names = (
+        "idx", "evo_dump_idx", 
+        "iter_num", "fid_func_call_num", "grad_func_call_num",
+        "num_amps_changed", "num_timeslots_changed",
+        "wall_time_dyn_gen_compute", "wall_time_prop_compute",
+        "wall_time_fwd_prop_compute", "wall_time_onwd_prop_compute")
+        
+    summary_property_fmt_type = (
+        'd', 'd',
+        'd', 'd', 'd',
+        'd', 'd',
+        'g', 'g', 
+        'g', 'g'
+        )
+        
+    summary_property_fmt_prec = (
+        0, 0, 
+        0, 0, 0,
+        0, 0, 
+        3, 3,
+        3, 3
+        )
+        
+    def __init__(self):
+        self.reset()
+        
+    def reset(self):
+        qtrldump.DumpSummaryItem.reset(self)
+        self.evo_dump_idx = None
+        self.iter_num = None
+        self.fid_func_call_num = None
+        self.grad_func_call_num = None
+        self.num_amps_changed = 0
+        self.num_timeslots_changed = 0
+        self.wall_time_dyn_gen_compute = 0.0
+        self.wall_time_prop_compute = 0.0
+        self.wall_time_fwd_prop_compute = 0.0
+        self.wall_time_onwd_prop_compute = 0.0
+        
\ No newline at end of file
