@@ -31,6 +31,8 @@
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
 
+__all__ = ['eseries', 'esval', 'esspec', 'estidy']
+
 import numpy as np
 import scipy.sparse as sp
 from qutip.qobj import Qobj
@@ -64,33 +66,36 @@ class eseries():
     """
     __array_priority__ = 101
 
-    def __init__(self, q=np.array([]), s=np.array([])):
+    def __init__(self, q=None, s=np.array([])):
+
         if isinstance(s, (int, float, complex)):
             s = np.array([s])
-        if (not np.any(q)) and (len(s) == 0):
+
+        if q is None:
             self.ampl = np.array([])
             self.rates = np.array([])
             self.dims = [[1, 1]]
             self.shape = [1, 1]
-        if np.any(q) and (len(s) == 0):
+
+        elif (len(s) == 0):
             if isinstance(q, eseries):
                 self.ampl = q.ampl
                 self.rates = q.rates
                 self.dims = q.dims
                 self.shape = q.shape
             elif isinstance(q, (np.ndarray, list)):
+                q = np.asarray(q, dtype=object)
                 ind = np.shape(q)
                 num = ind[0]  # number of elements in q
-                sh = np.array([Qobj(x).shape for x in q])
-                if any(sh != sh[0]):
+                if any([Qobj(x).shape != Qobj(q[0]).shape for x in q]):
                     raise TypeError('All amplitudes must have same dimension.')
-                self.ampl = np.array([x for x in q])
+                self.ampl = np.array([x for x in q], dtype=object)
                 self.rates = np.zeros(ind)
                 self.dims = self.ampl[0].dims
                 self.shape = self.ampl[0].shape
             elif isinstance(q, Qobj):
                 qo = Qobj(q)
-                self.ampl = np.array([qo])
+                self.ampl = np.array([qo], dtype=object)
                 self.rates = np.array([0])
                 self.dims = qo.dims
                 self.shape = qo.shape
@@ -100,21 +105,23 @@ class eseries():
                 self.dims = [[1, 1]]
                 self.shape = [1, 1]
 
-        if np.any(q) and len(s) != 0:
+        elif len(s) != 0:
             if isinstance(q, (np.ndarray, list)):
+                q = np.asarray(q, dtype=object)
                 ind = np.shape(q)
                 num = ind[0]
-                sh = np.array([Qobj(q[x]).shape for x in range(0, num)])
-                if np.any(sh != sh[0]):
+                if any([Qobj(x).shape != Qobj(q[0]).shape for x in q]):
                     raise TypeError('All amplitudes must have same dimension.')
-                self.ampl = np.array([Qobj(q[x]) for x in range(0, num)])
+                self.ampl = np.array([Qobj(q[x]) for x in range(0, num)],
+                                     dtype=object)
                 self.dims = self.ampl[0].dims
                 self.shape = self.ampl[0].shape
             else:
                 num = 1
-                self.ampl = np.array([Qobj(q)])
+                self.ampl = np.array([Qobj(q)], dtype=object)
                 self.dims = self.ampl[0].dims
                 self.shape = self.ampl[0].shape
+
             if isinstance(s, (int, complex, float)):
                 if num != 1:
                     raise TypeError('Number of rates must match number ' +
@@ -125,14 +132,16 @@ class eseries():
                     raise TypeError('Number of rates must match number ' +
                                     ' of members in object array.')
                 self.rates = np.array(s)
+
         if len(self.ampl) != 0:
             # combine arrays so that they can be sorted together
             zipped = list(zip(self.rates, self.ampl))
             zipped.sort()  # sort rates from lowest to highest
             rates, ampl = list(zip(*zipped))  # get back rates and ampl
-            self.ampl = np.array(ampl)
+            self.ampl = np.array(ampl, dtype=object)
             self.rates = np.array(rates)
 
+
     def __str__(self):  # string of ESERIES information
         self.tidyup()
         s = "ESERIES object: " + str(len(self.ampl)) + " terms\n"
@@ -237,7 +246,6 @@ class eseries():
             Values of exponential at times in ``tlist``.
 
         """
-
         if self.ampl is None or len(self.ampl) == 0:
             # no terms, evalue to zero
             return np.zeros(np.shape(tlist))
@@ -258,6 +266,7 @@ class eseries():
 
                 val_list.append(val)
 
+            val_list = np.array(val_list, dtype=object)
         else:
             # the amplitude vector contains c numbers
             val_list = np.zeros(np.size(tlist), dtype=complex)
@@ -333,7 +342,7 @@ class eseries():
         self.rates = np.array([])
         self.ampl = np.array([])
         for ur_key in unique_rates.keys():
-            total_ampl = np.sum(ampl_dict[ur_key])
+            total_ampl = np.sum(np.asarray(ampl_dict[ur_key], dtype=object))
 
             if (isinstance(total_ampl, float) or
                     isinstance(total_ampl, complex)):
@@ -343,7 +352,9 @@ class eseries():
             else:
                 if abs(total_ampl.full()).max() > ampl_tol:
                     self.rates = np.append(self.rates, unique_rates[ur_key])
-                    self.ampl = np.append(self.ampl, total_ampl)
+                    self.ampl = np.append(self.ampl,
+                                          np.asarray([total_ampl],
+                                                     dtype=object))
 
         return self
 
