@@ -31,12 +31,15 @@
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
 
+__all__ = ['expect', 'variance']
+
 import numpy as np
 import scipy.sparse as sp
 
-from qutip.qobj import Qobj, issuper, isoper
+from qutip.qobj import Qobj, isoper
 from qutip.eseries import eseries
-from qutip.cy.spmatfuncs import (cy_expect_rho_vec, cy_expect_psi, cy_spmm_tr)
+from qutip.cy.spmatfuncs import (cy_expect_rho_vec, cy_expect_psi, cy_spmm_tr,
+                                expect_csr_ket)
 
 
 expect_rho_vec = cy_expect_rho_vec
@@ -112,7 +115,7 @@ def _single_qobj_expect(oper, state):
 
         elif state.type == 'ket':
             # calculates expectation value via <psi|op|psi>
-            return cy_expect_psi(oper.data, state.full(squeeze=True),
+            return expect_csr_ket(oper.data, state.data,
                                  oper.isherm)
     else:
         raise TypeError('Invalid operand types')
