@@ -35,7 +35,7 @@ import operator
 from scipy.sparse import (_sparsetools, isspmatrix, isspmatrix_csr,
                           csr_matrix, coo_matrix, csc_matrix, dia_matrix)
 from scipy.sparse.sputils import (upcast, upcast_char, to_native, isdense, isshape,
-                      getdtype, isscalarlike, IndexMixin, get_index_dtype)
+                      getdtype, isscalarlike, get_index_dtype)
 from scipy.sparse.base import spmatrix, isspmatrix, SparseEfficiencyWarning
 from warnings import warn
 
@@ -52,7 +52,7 @@ class fast_csr_matrix(csr_matrix):
             self.indices = np.array([], dtype=np.int32)
             self.indptr = np.zeros(shape[0]+1, dtype=np.int32)
             self._shape = tuple(int(s) for s in shape)
-            
+
         else:
             if args[0].shape[0] and args[0].dtype != complex:
                 raise TypeError('fast_csr_matrix allows only complex data.')
@@ -73,7 +73,7 @@ class fast_csr_matrix(csr_matrix):
 
     def _binopt(self, other, op):
         """
-        Do the binary operation fn to two sparse matrices using 
+        Do the binary operation fn to two sparse matrices using
         fast_csr_matrix only when other is also a fast_csr_matrix.
         """
         # e.g. csr_plus_csr, csr_minus_csr, etc.
@@ -116,7 +116,7 @@ class fast_csr_matrix(csr_matrix):
         else:
             A = csr_matrix((data, indices, indptr), dtype=data.dtype, shape=self.shape)
         return A
-    
+
     def multiply(self, other):
         """Point-wise multiplication by another matrix, vector, or
         scalar.
@@ -174,7 +174,7 @@ class fast_csr_matrix(csr_matrix):
                 return self._mul_scalar(other.flat[0])
         # Anything else.
         return np.multiply(self.todense(), other)
-    
+
     def _mul_sparse_matrix(self, other):
         """
         Do the sparse matrix mult returning fast_csr_matrix only
@@ -187,7 +187,7 @@ class fast_csr_matrix(csr_matrix):
         if isinstance(other, fast_csr_matrix):
             A = zcsr_mult(self, other, sorted=1)
             return A
-        
+
         other = csr_matrix(other)  # convert to this format
         idx_dtype = get_index_dtype((self.indptr, self.indices,
                                      other.indptr, other.indices),
@@ -325,7 +325,7 @@ class fast_csr_matrix(csr_matrix):
             return all_true - res
         else:
             raise ValueError("Operands could not be compared.")
-        
+
     def _with_data(self,data,copy=True):
         """Returns a matrix with the same sparsity structure as self,
         but with different data.  By default the structure arrays
@@ -340,33 +340,33 @@ class fast_csr_matrix(csr_matrix):
         else:
             return fast_csr_matrix((data,self.indices,self.indptr),
                                    shape=self.shape,dtype=data.dtype)
-    
+
     def transpose(self):
         """
         Returns the transpose of the matrix, keeping
         it in fast_csr format.
         """
         return zcsr_transpose(self)
-    
+
     def trans(self):
         """
         Same as transpose
         """
         return zcsr_transpose(self)
-    
+
     def getH(self):
         """
         Returns the conjugate-transpose of the matrix, keeping
         it in fast_csr format.
         """
         return zcsr_adjoint(self)
-    
+
     def adjoint(self):
         """
         Same as getH
         """
         return zcsr_adjoint(self)
-    
+
 
 def csr2fast(A, copy=False):
     if (not isinstance(A, fast_csr_matrix)) or copy:
@@ -403,4 +403,4 @@ def _all_true(shape):
 
 #Need to do some trailing imports here
 #-------------------------------------
-from qutip.cy.spmath import (zcsr_transpose, zcsr_adjoint, zcsr_mult)
\ No newline at end of file
+from qutip.cy.spmath import (zcsr_transpose, zcsr_adjoint, zcsr_mult)
