@@ -35,8 +35,9 @@ This module contains a collection of graph theory routines used mainly
 to reorder matrices for iterative steady state solvers.
 """
 
-__all__ = ['graph_degree', 'breadth_first_search', 'reverse_cuthill_mckee',
-           'maximum_bipartite_matching', 'weighted_bipartite_matching']
+__all__ = ['graph_degree', 'column_permutation', 'breadth_first_search', 
+            'reverse_cuthill_mckee', 'maximum_bipartite_matching', 
+            'weighted_bipartite_matching']
 
 import numpy as np
 import scipy.sparse as sp
@@ -64,7 +65,7 @@ def graph_degree(A):
     """
     if not (sp.isspmatrix_csc(A) or sp.isspmatrix_csr(A)):
         raise TypeError('Input must be CSC or CSR sparse matrix.')
-    return _node_degrees(A.indices, A.indptr, A.shape[0])
+    return np.asarray(_node_degrees(A.indices, A.indptr, A.shape[0]))
 
 
 def breadth_first_search(A, start):
@@ -101,6 +102,31 @@ def breadth_first_search(A, start):
     return order[order != -1], levels[levels != -1]
 
 
+def column_permutation(A):
+    """
+    Finds the non-symmetric column permutation of A such that the columns 
+    are given in ascending order according to the number of nonzero entries.
+    This is sometimes useful for decreasing the fill-in of sparse LU 
+    factorization.
+    
+    Parameters
+    ----------
+    A : csc_matrix
+        Input sparse CSC sparse matrix.
+
+    Returns
+    -------
+    perm : array
+        Array of permuted row and column indices.
+    
+    """
+    if not sp.isspmatrix_csc(A):
+        A = sp.csc_matrix(A)
+    count = np.diff(A.indptr)
+    perm = np.argsort(count)
+    return perm
+
+
 def reverse_cuthill_mckee(A, sym=False):
     """
     Returns the permutation array that orders a sparse CSR or CSC matrix
@@ -136,6 +162,7 @@ def reverse_cuthill_mckee(A, sym=False):
     E. Cuthill and J. McKee, "Reducing the Bandwidth of Sparse Symmetric
     Matrices", ACM '69 Proceedings of the 1969 24th national conference,
     (1969).
+    
     """
     if not (sp.isspmatrix_csc(A) or sp.isspmatrix_csr(A)):
         raise TypeError('Input must be CSC or CSR sparse matrix.')
@@ -178,10 +205,10 @@ def maximum_bipartite_matching(A, perm_type='row'):
 
     References
     ----------
-    .. [1] I. S. Duff, K. Kaya, and B. Ucar, "Design, Implementation, and
+    I. S. Duff, K. Kaya, and B. Ucar, "Design, Implementation, and
     Analysis of Maximum Transversal Algorithms", ACM Trans. Math. Softw.
     38, no. 2, (2011).
-
+    
     """
     nrows = A.shape[0]
     if A.shape[0] != A.shape[1]:
@@ -240,11 +267,12 @@ def weighted_bipartite_matching(A, perm_type='row'):
 
     References
     ----------
-    .. [1] I. S. Duff and J. Koster, "The design and use of algorithms for
+    I. S. Duff and J. Koster, "The design and use of algorithms for
     permuting large entries to the diagonal of sparse matrices", SIAM J.
     Matrix Anal. and Applics. 20, no. 4, 889 (1997).
 
     """
+   
     nrows = A.shape[0]
     if A.shape[0] != A.shape[1]:
         raise ValueError('weighted_bfs_matching requires a square matrix.')
@@ -258,8 +286,8 @@ def weighted_bipartite_matching(A, perm_type='row'):
         A = A.transpose().tocsc()
 
     perm = _weighted_bipartite_matching(
-        np.asarray(np.abs(A.data), dtype=float),
-        A.indices, A.indptr, nrows)
+                    np.asarray(np.abs(A.data), dtype=float), 
+                    A.indices, A.indptr, nrows)
 
     if np.any(perm == -1):
         raise Exception('Possibly singular input matrix.')
