@@ -101,7 +101,7 @@ shape = [3, 3], type = oper, isHerm = True
         raise TypeError('j must be a non-negative integer or half-integer')
 
     if not args:
-        return jmat(j, 'x'), jmat(j, 'y'), jmat(j, 'z') 
+        return jmat(j, 'x'), jmat(j, 'y'), jmat(j, 'z')
 
     if args[0] == '+':
         A = _jplus(j)
@@ -266,7 +266,7 @@ def sigmap():
 
     Examples
     --------
-    >>> sigmam()
+    >>> sigmap()
     Quantum object: dims = [[2], [2]], \
 shape = [2, 2], type = oper, isHerm = False
     Qobj data =
@@ -458,7 +458,7 @@ shape = [3, 3], type = oper, isHerm = True
     N = int(N)
     if N < 0:
         raise ValueError("N must be integer N>=0")
-    return Qobj(fast_identity(N), isherm=True)
+    return Qobj(fast_identity(N), isherm=True, isunitary=True)
 
 
 def identity(N):
@@ -562,7 +562,7 @@ shape = [4, 4], type = oper, isHerm = True
         ind = np.arange(N, dtype=np.int32)
         ptr = np.arange(N+1,dtype=np.int32)
         ptr[-1] = N
-    
+
     return Qobj(fast_csr_matrix((data,ind,ptr), shape=(N,N)), isherm=True)
 
 
@@ -717,6 +717,7 @@ def qdiags(diagonals, offsets, dims=None, shape=None):
     ----------
     diagonals : sequence of array_like
         Array of elements to place along the selected diagonals.
+
     offsets : sequence of ints
         Sequence for diagonals to be set:
             - k=0 main diagonal
@@ -724,13 +725,14 @@ def qdiags(diagonals, offsets, dims=None, shape=None):
             - k<0 kth lower diagonal
     dims : list, optional
         Dimensions for operator
+
     shape : list, tuple, optional
         Shape of operator.  If omitted, a square operator large enough
         to contain the diagonals is generated.
 
     See Also
     --------
-    scipy.sparse.diags for usage information.
+    scipy.sparse.diags : for usage information.
 
     Notes
     -----
@@ -738,7 +740,7 @@ def qdiags(diagonals, offsets, dims=None, shape=None):
 
     Examples
     --------
-    >>> qdiags(sqrt(range(1,4)),1)
+    >>> qdiags(sqrt(range(1, 4)), 1)
     Quantum object: dims = [[4], [4]], \
 shape = [4, 4], type = oper, isherm = False
     Qobj data =
@@ -748,10 +750,7 @@ shape = [4, 4], type = oper, isherm = False
      [ 0.          0.          0.          0.        ]]
 
     """
-    try:
-        data = sp.diags(diagonals, offsets, shape, format='csr', dtype=complex)
-    except:
-        raise NotImplementedError("This function requires SciPy 0.11+.")
+    data = sp.diags(diagonals, offsets, shape, format='csr', dtype=complex)
     if not dims:
         dims = [[], []]
     if not shape:
@@ -911,27 +910,27 @@ def charge(Nmax, Nmin=None, frac = 1):
     """
     Generate the diagonal charge operator over charge states
     from Nmin to Nmax.
-    
+
     Parameters
     ----------
     Nmax : int
         Maximum charge state to consider.
-    
+
     Nmin : int (default = -Nmax)
         Lowest charge state to consider.
-    
+
     frac : float (default = 1)
         Specify fractional charge if needed.
-    
+
     Returns
     -------
     C : Qobj
         Charge operator over [Nmin,Nmax].
-    
+
     Notes
     -----
     .. versionadded:: 3.2
-    
+
     """
     if Nmin is None:
         Nmin = -Nmax
@@ -945,25 +944,25 @@ def charge(Nmax, Nmin=None, frac = 1):
 
 def tunneling(N, m=1):
     """
-    Tunneling operator with elements of the form 
+    Tunneling operator with elements of the form
     :math:`\sum |N><N+m| + |N+m><N|`.
-    
+
     Parameters
     ----------
     N : int
         Number of basis states in Hilbert space.
     m : int (default = 1)
         Number of excitations in tunneling event.
-    
+
     Returns
     -------
     T : Qobj
         Tunneling operator.
-    
+
     Notes
     -----
     .. versionadded:: 3.2
-    
+
     """
     diags = [np.ones(N-m,dtype=int),np.ones(N-m,dtype=int)]
     T = sp.diags(diags,[m,-m],format='csr', dtype=complex)
