@@ -55,8 +55,9 @@ from qutip.ui.progressbar import BaseProgressBar, TextProgressBar
 
 
 def propagator(H, t, c_op_list=[], args={}, options=None,
-               unitary_mode='batch', parallel=False, 
-               progress_bar=None, **kwargs):
+               unitary_mode='batch', parallel=False,
+               progress_bar=None, _safe_mode=True,
+               **kwargs):
     """
     Calculate the propagator U(t) for the density matrix or wave function such
     that :math:`\psi(t) = U(t)\psi(0)` or
@@ -85,13 +86,13 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
         with options for the ODE solver.
 
     unitary_mode = str ('batch', 'single')
-        Solve all basis vectors simulaneously ('batch') or individually 
+        Solve all basis vectors simulaneously ('batch') or individually
         ('single').
-    
+
     parallel : bool {False, True}
-        Run the propagator in parallel mode. This will override the 
+        Run the propagator in parallel mode. This will override the
         unitary_mode settings if set to True.
-    
+
     progress_bar: BaseProgressBar
         Optional instance of BaseProgressBar, or a subclass thereof, for
         showing the progress of the simulation. By default no progress bar
@@ -108,7 +109,7 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
         num_cpus = kwargs['num_cpus']
     else:
         num_cpus = kw['num_cpus']
-    
+
     if progress_bar is None:
         progress_bar = BaseProgressBar()
     elif progress_bar is True:
@@ -124,22 +125,28 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
     else:
         tlist = t
 
+    if _safe_mode:
+        _solver_safety_check(H, None, c_ops=c_op_list, e_ops=[], args=args)
+
     td_type = _td_format_check(H, c_op_list, solver='me')
-        
+
     if isinstance(H, (types.FunctionType, types.BuiltinFunctionType,
                       functools.partial)):
         H0 = H(0.0, args)
+        if unitary_mode =='batch':
+            # batch don't work with function Hamiltonian
+            unitary_mode = 'single'
     elif isinstance(H, list):
         H0 = H[0][0] if isinstance(H[0], list) else H[0]
     else:
         H0 = H
-    
+
     if len(c_op_list) == 0 and H0.isoper:
         # calculate propagator for the wave function
 
         N = H0.shape[0]
         dims = H0.dims
-        
+
         if parallel:
             unitary_mode = 'single'
             u = np.zeros([N, N, len(tlist)], dtype=complex)
@@ -148,20 +155,15 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
                                   progress_bar=progress_bar, num_cpus=num_cpus)
             for n in range(N):
                 for k, t in enumerate(tlist):
-                    u[:, n, k] = output[n].states[k].full().T 
+                    u[:, n, k] = output[n].states[k].full().T
         else:
             if unitary_mode == 'single':
-                u = np.zeros([N, N, len(tlist)], dtype=complex)
-                progress_bar.start(N)
-                for n in range(0, N):
-                    progress_bar.update(n)
-                    psi0 = basis(N, n)
-                    output = sesolve(H, psi0, tlist, [], args, options,
-                                     _safe_mode=False)
-                    for k, t in enumerate(tlist):
-                        u[:, n, k] = output.states[k].full().T
-                    progress_bar.finished()
-
+                output = sesolve(H, qeye(N), tlist, [], args, options,
+                                 _safe_mode=False)
+                if len(tlist) == 2:
+                    return output.states[-1]
+                else:
+                    return output.states
 
             elif unitary_mode =='batch':
                 u = np.zeros(len(tlist), dtype=object)
@@ -178,9 +180,10 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
                             H2.append(tensor(qeye(N), H[k]))
                 else:
                     H2 = tensor(qeye(N), H)
+                options.normalize_output = False
                 output = sesolve(H2, psi0, tlist, [],
-                                 args=args, _safe_mode=False,
-                                 options=Options(normalize_output=False))
+                                 args=args, options=options,
+                                 _safe_mode=False)
                 for k, t in enumerate(tlist):
                     u[k] = sp_reshape(output.states[k].data, (N, N))
                     unit_row_norm(u[k].data, u[k].indptr, u[k].shape[0])
@@ -188,7 +191,7 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
 
             else:
                 raise Exception('Invalid unitary mode.')
-                        
+
 
     elif len(c_op_list) == 0 and H0.issuper:
         # calculate the propagator for the vector representation of the
@@ -197,7 +200,7 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
         N = H0.shape[0]
         sqrt_N = int(np.sqrt(N))
         dims = H0.dims
-        
+
         u = np.zeros([N, N, len(tlist)], dtype=complex)
 
         if parallel:
@@ -210,18 +213,14 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
                 for k, t in enumerate(tlist):
                     u[:, n, k] = mat2vec(output[n].states[k].full()).T
         else:
-            progress_bar.start(N)
-            for n in range(0, N):
-                progress_bar.update(n)
-                col_idx, row_idx = np.unravel_index(n, (sqrt_N, sqrt_N))
-                rho0 = Qobj(sp.csr_matrix(([1], ([row_idx], [col_idx])),
-                                          shape=(sqrt_N,sqrt_N), dtype=complex)
-                            )
-                output = mesolve(H, rho0, tlist, [], [], args, options,
-                                 _safe_mode=False)
-                for k, t in enumerate(tlist):
-                    u[:, n, k] = mat2vec(output.states[k].full()).T
-            progress_bar.finished()
+            rho0 = qeye(N,N)
+            rho0.dims = [[sqrt_N, sqrt_N], [sqrt_N, sqrt_N]]
+            output = mesolve(H, psi0, tlist, [], args, options,
+                             _safe_mode=False)
+            if len(tlist) == 2:
+                return output.states[-1]
+            else:
+                return output.states
 
     else:
         # calculate the propagator for the vector representation of the
@@ -231,7 +230,7 @@ def propagator(H, t, c_op_list=[], args={}, options=None,
         dims = [H0.dims, H0.dims]
 
         u = np.zeros([N * N, N * N, len(tlist)], dtype=complex)
-        
+
         if parallel:
             output = parallel_map(_parallel_mesolve, range(N * N),
                                   task_args=(
@@ -295,7 +294,7 @@ def propagator_steadystate(U):
     """
 
     evals, evecs = la.eig(U.full())
-    
+
     shifted_vals = np.abs(evals - 1.0)
     ev_idx = np.argmin(shifted_vals)
     ev_min = shifted_vals[ev_idx]
@@ -319,4 +318,3 @@ def _parallel_mesolve(n, N, H, tlist, c_op_list, args, options):
     output = mesolve(H, rho0, tlist, c_op_list, [], args, options,
                      _safe_mode=False)
     return output
-
