@@ -31,75 +31,85 @@
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
 
-__all__ = ['Gate', 'QubitCircuit']
+from collections.abc import Iterable
+import warnings
+import inspect
+
+import numpy as np
 
 import numpy as np
-import warnings
 
 from qutip.qip.circuit_latex import _latex_compile
 from qutip.qip.gates import *
+from qutip.qip.qubits import qubit_states
+
+__all__ = ['Gate', 'QubitCircuit']
 
 
 class Gate(object):
     """
     Representation of a quantum gate, with its required parametrs, and target
     and control qubits.
+
+    Parameters
+    ----------
+    name : string
+        Gate name.
+    targets : list or int
+        Gate targets.
+    controls : list or int
+        Gate controls.
+    arg_value : float
+        Argument value(phi).
+    arg_label : string
+        Label for gate representation.
     """
 
     def __init__(self, name, targets=None, controls=None, arg_value=None,
                  arg_label=None):
         """
-        Creates a gate with specified parameters.
-
-        Parameters
-        ----------
-        name: String
-            Gate name.
-        targets: List
-            Gate targets.
-        controls: List
-            Gate controls.
-        arg_value: Float
-            Argument value(phi).
-        arg_label: String
-            Label for gate representation.
+        Create a gate with specified parameters.
         """
         self.name = name
         self.targets = None
         self.controls = None
 
-        if not isinstance(targets, list) and targets is not None:
+        if not isinstance(targets, Iterable) and targets is not None:
             self.targets = [targets]
         else:
             self.targets = targets
 
-        if not isinstance(controls, list) and controls is not None:
+        if not isinstance(controls, Iterable) and controls is not None:
             self.controls = [controls]
         else:
             self.controls = controls
 
-        self.arg_value = arg_value
-        self.arg_label = arg_label
+        for ind_list in [self.targets, self.controls]:
+            if isinstance(ind_list, Iterable):
+                all_integer = all(
+                    [isinstance(ind, np.int) for ind in ind_list])
+                if not all_integer:
+                    raise ValueError("Index of a qubit must be an integer")
 
         if name in ["SWAP", "ISWAP", "SQRTISWAP", "SQRTSWAP", "BERKELEY",
                     "SWAPalpha"]:
-            if len(self.targets) != 2:
-                raise ValueError("Gate %s requires two target" % name)
+            if (self.targets is None) or (len(self.targets) != 2):
+                raise ValueError("Gate %s requires two targets" % name)
             if self.controls is not None:
-                raise ValueError("Gate %s does not require a control" % name)
+                raise ValueError("Gate %s cannot have a control" % name)
 
-        if name in ["CNOT", "CSIGN", "CRX", "CRY", "CRZ"]:
+        elif name in ["CNOT", "CSIGN", "CRX", "CRY", "CRZ"]:
             if self.targets is None or len(self.targets) != 1:
                 raise ValueError("Gate %s requires one target" % name)
             if self.controls is None or len(self.controls) != 1:
                 raise ValueError("Gate %s requires one control" % name)
 
-        if name in ["SNOT", "RX", "RY", "RZ", "PHASEGATE"]:
+        elif name in ["SNOT", "RX", "RY", "RZ", "PHASEGATE"]:
             if self.controls is not None:
                 raise ValueError("Gate %s does not take controls" % name)
 
-        if name in ["RX", "RY", "RZ", "CPHASE", "SWAPalpha", "PHASEGATE",
-                    "GLOBALPHASE", "CRX", "CRY", "CRZ"]:
+        elif name in ["RX", "RY", "RZ", "CPHASE", "SWAPalpha", "PHASEGATE",
+                            "GLOBALPHASE", "CRX", "CRY", "CRZ"]:
             if arg_value is None:
                 raise ValueError("Gate %s requires an argument value" % name)
 
@@ -162,36 +172,113 @@ class QubitCircuit(object):
     """
     Representation of a quantum program/algorithm, maintaining a sequence
     of gates.
+
+    Parameters
+    ----------
+    N : int
+        Number of qubits in the system.
+    user_gates : dict
+        Define a dictionary of the custom gates. See examples for detail.
+    input_states : list
+        A list of string such as `0`,'+', "A", "Y". Only used for latex.
+
+    Examples
+    --------
+    >>> def user_gate():
+    ...     mat = np.array([[1.,   0],
+    ...                     [0., 1.j]])
+    ...     return Qobj(mat, dims=[[2], [2]])
+    >>> qc.QubitCircuit(2, user_gates={"T":user_gate})
+    >>> qc.add_gate("T", targets=[0])
     """
 
-    def __init__(self, N, reverse_states=True):
-
+    def __init__(self, N, input_states=None, output_states=None,
+                 reverse_states=True, user_gates=None):
         # number of qubits in the register
         self.N = N
         self.reverse_states = reverse_states
         self.gates = []
         self.U_list = []
+        self.input_states = [None for i in range(N)]
+        self.output_states = [None for i in range(N)]
+        if user_gates is None:
+            self.user_gates = {}
+        else:
+            if isinstance(user_gates, dict):
+                self.user_gates = user_gates
+            else:
+                raise ValueError(
+                    "`user_gate` takes a python dictionary of the form"
+                    "{{str: gate_function}}, not {}".format(user_gates))
 
-    def add_gate(self, name, targets=None, controls=None, arg_value=None,
-                 arg_label=None):
+    def add_state(self, state, targets=None, state_type="input"):
+        """
+        Add an input or ouput state to the circuit. By default all the input
+        and output states will be initialized to `None`. A particular state can
+        be added by specifying the state and the qubit where it has to be added
+        along with the type as input or output.
+
+        Parameters
+        ----------
+        state: str
+            The state that has to be added. It can be any string such as `0`,
+            '+', "A", "Y"
+        targets: list
+            A list of qubit positions where the given state has to be added.
+        state_type: str
+            One of either "input" or "output". This specifies whether the state
+            to be added is an input or output.
+            default: "input"
+
+        """
+        if state_type == "input":
+            for i in targets:
+                self.input_states[i] = state
+        if state_type == "output":
+            for i in targets:
+                self.output_states[i] = state
+
+    def add_gate(self, gate, targets=None, controls=None, arg_value=None,
+                 arg_label=None, index=None):
         """
         Adds a gate with specified parameters to the circuit.
 
         Parameters
         ----------
-        name: String
-            Gate name.
-        targets: List
+        gate: string or `Gate`
+            Gate name. If gate is an instance of `Gate`, parameters are
+            unpacked and added.
+        targets: list
             Gate targets.
-        controls: List
+        controls: list
             Gate controls.
-        arg_value: Float
+        arg_value: float
             Argument value(phi).
-        arg_label: String
+        arg_label: string
             Label for gate representation.
+        index : list
+            Positions to add the gate.
         """
-        self.gates.append(Gate(name, targets=targets, controls=controls,
-                               arg_value=arg_value, arg_label=arg_label))
+        if isinstance(gate, Gate):
+            name = gate.name
+            targets = gate.targets
+            controls = gate.controls
+            arg_value = gate.arg_value
+            arg_label = gate.arg_label
+
+        else:
+            name = gate
+
+        if index is None:
+            self.gates.append(Gate(name, targets=targets, controls=controls,
+                                   arg_value=arg_value, arg_label=arg_label))
+
+        else:
+            for position in index:
+                self.gates.insert(position, Gate(name, targets=targets,
+                                                 controls=controls,
+                                                 arg_value=arg_value,
+                                                 arg_label=arg_label))
 
     def add_1q_gate(self, name, start=0, end=None, qubits=None,
                     arg_value=None, arg_label=None):
@@ -202,17 +289,17 @@ class QubitCircuit(object):
 
         Parameters
         ----------
-        name: String
+        name : string
             Gate name.
-        start: Integer
+        start : int
             Starting location of qubits.
-        end: Integer
+        end : int
             Last qubit for the gate.
-        qubits: List
+        qubits : list
             Specific qubits for applying gates.
-        arg_value: Float
+        arg_value : float
             Argument value(phi).
-        arg_label: String
+        arg_label : string
             Label for gate representation.
         """
         if name not in ["RX", "RY", "RZ", "SNOT", "SQRTNOT", "PHASEGATE"]:
@@ -239,13 +326,12 @@ class QubitCircuit(object):
 
         Parameters
         ----------
-        qc: QubitCircuit
+        qc : QubitCircuit
             The circuit block to be added to the main circuit.
-        start: Integer
+        start : int
             The qubit on which the first gate is applied.
         """
-
-        if self.N - start < len(qc.gates):
+        if self.N - start < qc.N:
             raise NotImplementedError("Targets exceed number of qubits.")
 
         for gate in qc.gates:
@@ -270,23 +356,33 @@ class QubitCircuit(object):
                               [gate.targets[0] + start,
                                gate.targets[1] + start],
                               gate.controls + start, None, None)
+            elif gate.name in self.user_gates:
+                self.add_gate(
+                              gate.name, targets=gate.targets,
+                              arg_value=gate.arg_value)
 
-    def remove_gate(self, index=None, name=None, remove="first"):
+    def remove_gate(self, index=None, end=None, name=None, remove="first"):
         """
-        Removes a gate with from a specific index or the first, last or all
-        instances of a particular gate.
+        Remove a gate from a specific index or between two indexes or the
+        first, last or all instances of a particular gate.
 
         Parameters
         ----------
-        index: Integer
+        index : int
             Location of gate to be removed.
-        name: String
+        name : string
             Gate name to be removed.
-        remove: String
+        remove : string
             If first or all gate are to be removed.
         """
         if index is not None and index <= self.N:
-            self.gates.pop(index)
+            if end is not None and end <= self.N:
+                for i in range(end - index):
+                    self.gates.pop(index + i)
+            elif end is not None and end > self.N:
+                raise ValueError("End target exceeds number of gates.")
+            else:
+                self.gates.pop(index)
 
         elif name is not None and remove == "first":
             for gate in self.gates:
@@ -310,18 +406,19 @@ class QubitCircuit(object):
 
     def reverse_circuit(self):
         """
-        Reverses an entire circuit of unitary gates.
+        Reverse an entire circuit of unitary gates.
 
         Returns
         ----------
-        qc: QubitCircuit
-            Returns QubitCircuit of resolved gates for the qubit circuit in the
-            desired basis.
+        qc : QubitCircuit
+            Return QubitCircuit of resolved gates for the qubit circuit in the
+            reverse order.
+
         """
         temp = QubitCircuit(self.N, self.reverse_states)
 
-        for i in range(self.N):
-            temp.append(self.gates[self.N - i - 1])
+        for gate in reversed(self.gates):
+            temp.add_gate(gate)
 
         return temp
 
@@ -333,46 +430,41 @@ class QubitCircuit(object):
 
         Parameters
         ----------
-        basis: list.
+        basis : list.
             Basis of the resolved circuit.
 
         Returns
         -------
-        qc: QubitCircuit
-            Returns QubitCircuit of resolved gates for the qubit circuit in the
+        qc : QubitCircuit
+            Return QubitCircuit of resolved gates for the qubit circuit in the
             desired basis.
         """
         qc_temp = QubitCircuit(self.N, self.reverse_states)
         temp_resolved = []
 
-        basis_1q = []
-        basis_2q = None
-
         basis_1q_valid = ["RX", "RY", "RZ"]
         basis_2q_valid = ["CNOT", "CSIGN", "ISWAP", "SQRTSWAP", "SQRTISWAP"]
 
         if isinstance(basis, list):
+            basis_1q = []
+            basis_2q = []
             for gate in basis:
-                if gate not in (basis_1q_valid + basis_2q_valid):
-                    raise ValueError("%s is not a valid basis gate" % gate)
-
                 if gate in basis_2q_valid:
-                    if basis_2q is not None:
-                        raise ValueError("At most one two-qubit gate allowed")
-                    basis_2q = gate
-
-                else:
+                    basis_2q.append(gate)
+                elif gate in basis_1q_valid:
                     basis_1q.append(gate)
-
+                else:
+                    raise NotImplementedError(
+                        "%s is not a valid basis gate" % gate)
             if len(basis_1q) == 1:
                 raise ValueError("Not sufficient single-qubit gates in basis")
             elif len(basis_1q) == 0:
                 basis_1q = ["RX", "RY", "RZ"]
 
-        else:
+        else:  # only one 2q gate is given as basis
             basis_1q = ["RX", "RY", "RZ"]
             if basis in basis_2q_valid:
-                basis_2q = basis
+                basis_2q = [basis]
             else:
                 raise ValueError("%s is not a valid two-qubit basis gate"
                                  % basis)
@@ -406,11 +498,13 @@ class QubitCircuit(object):
                                           arg_label=gate.arg_label))
                 temp_resolved.append(Gate("RZ", gate.targets, None,
                                           gate.arg_value, gate.arg_label))
+            elif gate.name in basis_2q:  # ignore all gate in 2q basis
+                temp_resolved.append(gate)
             elif gate.name == "CPHASE":
                 raise NotImplementedError("Cannot be resolved in this basis")
             elif gate.name == "CNOT":
                 temp_resolved.append(gate)
-            elif gate.name == "CSIGN" and basis_2q is not "CSIGN":
+            elif gate.name == "CSIGN":
                 temp_resolved.append(Gate("RY", gate.targets, None,
                                           arg_value=np.pi / 2,
                                           arg_label=r"\pi/2"))
@@ -428,14 +522,17 @@ class QubitCircuit(object):
                 raise NotImplementedError("Cannot be resolved in this basis")
             elif gate.name == "SWAPalpha":
                 raise NotImplementedError("Cannot be resolved in this basis")
-            elif gate.name == "SWAP" and basis_2q is not "ISWAP":
-                temp_resolved.append(Gate("CNOT", gate.targets[0],
-                                          gate.targets[1]))
-                temp_resolved.append(Gate("CNOT", gate.targets[1],
-                                          gate.targets[0]))
-                temp_resolved.append(Gate("CNOT", gate.targets[0],
-                                          gate.targets[1]))
-            elif gate.name == "ISWAP" and basis_2q is not "ISWAP":
+            elif gate.name == "SWAP":
+                if "ISWAP" in basis_2q:  # dealed with separately
+                    temp_resolved.append(gate)
+                else:
+                    temp_resolved.append(
+                        Gate("CNOT", gate.targets[0], gate.targets[1]))
+                    temp_resolved.append(
+                        Gate("CNOT", gate.targets[1], gate.targets[0]))
+                    temp_resolved.append(
+                        Gate("CNOT", gate.targets[0], gate.targets[1]))
+            elif gate.name == "ISWAP":
                 temp_resolved.append(Gate("CNOT", gate.targets[0],
                                           gate.targets[1]))
                 temp_resolved.append(Gate("CNOT", gate.targets[1],
@@ -465,11 +562,9 @@ class QubitCircuit(object):
                 temp_resolved.append(Gate("GLOBALPHASE", None, None,
                                           arg_value=np.pi / 2,
                                           arg_label=r"\pi/2"))
-            elif gate.name == "SQRTSWAP" and basis_2q not in ["SQRTSWAP",
-                                                              "ISWAP"]:
+            elif gate.name == "SQRTSWAP":
                 raise NotImplementedError("Cannot be resolved in this basis")
-            elif gate.name == "SQRTISWAP" and basis_2q not in ["SQRTISWAP",
-                                                               "ISWAP"]:
+            elif gate.name == "SQRTISWAP":
                 raise NotImplementedError("Cannot be resolved in this basis")
             elif gate.name == "FREDKIN":
                 temp_resolved.append(Gate("CNOT", gate.targets[0],
@@ -600,9 +695,11 @@ class QubitCircuit(object):
                                           gate.controls,
                                           gate.arg_value, gate.arg_label))
             else:
-                temp_resolved.append(gate)
+                raise NotImplementedError(
+                    "Gate {} "
+                    "cannot be resolved.".format(gate.name))
 
-        if basis_2q == "CSIGN":
+        if "CSIGN" in basis_2q:
             for gate in temp_resolved:
                 if gate.name == "CNOT":
                     qc_temp.gates.append(Gate("RY", gate.targets, None,
@@ -615,7 +712,7 @@ class QubitCircuit(object):
                                               arg_label=r"\pi/2"))
                 else:
                     qc_temp.gates.append(gate)
-        elif basis_2q == "ISWAP":
+        elif "ISWAP" in basis_2q:
             for gate in temp_resolved:
                 if gate.name == "CNOT":
                     qc_temp.gates.append(Gate("GLOBALPHASE", None, None,
@@ -662,7 +759,7 @@ class QubitCircuit(object):
                                               arg_label=r"-\pi/2"))
                 else:
                     qc_temp.gates.append(gate)
-        elif basis_2q == "SQRTSWAP":
+        elif "SQRTSWAP" in basis_2q:
             for gate in temp_resolved:
                 if gate.name == "CNOT":
                     qc_temp.gates.append(Gate("RY", gate.targets, None,
@@ -688,7 +785,7 @@ class QubitCircuit(object):
                                               arg_label=r"-\pi/2"))
                 else:
                     qc_temp.gates.append(gate)
-        elif basis_2q == "SQRTISWAP":
+        elif "SQRTISWAP" in basis_2q:
             for gate in temp_resolved:
                 if gate.name == "CNOT":
                     qc_temp.gates.append(Gate("RY", gate.controls, None,
@@ -768,13 +865,13 @@ class QubitCircuit(object):
         target/s in terms of gates with adjacent interactions.
 
         Returns
-        ----------
-        qc: QubitCircuit
-            Returns QubitCircuit of resolved gates for the qubit circuit in the
-            desired basis.
+        -------
+        qc : QubitCircuit
+            Return QubitCircuit of the gates for the qubit circuit with the
+            resolved non-adjacent gates.
 
         """
-        temp = QubitCircuit(self.N, self.reverse_states)
+        temp = QubitCircuit(self.N, reverse_states=self.reverse_states)
         swap_gates = ["SWAP", "ISWAP", "SQRTISWAP", "SQRTSWAP", "BERKELEY",
                       "SWAPalpha"]
 
@@ -839,7 +936,9 @@ class QubitCircuit(object):
                     i += 1
 
             else:
-                temp.gates.append(gate)
+                raise NotImplementedError(
+                    "`adjacent_gates` is not defined for "
+                    "gate {}.".format(gate.name))
 
         return temp
 
@@ -850,8 +949,8 @@ class QubitCircuit(object):
 
         Returns
         -------
-        U_list: list
-            Returns list of unitary matrices for the qubit circuit.
+        U_list : list
+            Return list of unitary matrices for the qubit circuit.
 
         """
         self.U_list = []
@@ -870,7 +969,7 @@ class QubitCircuit(object):
             elif gate.name == "PHASEGATE":
                 self.U_list.append(phasegate(gate.arg_value, self.N,
                                              gate.targets[0]))
-            if gate.name == "CRX":
+            elif gate.name == "CRX":
                 self.U_list.append(controlled_gate(rx(gate.arg_value),
                                                    N=self.N,
                                                    control=gate.controls[0],
@@ -915,6 +1014,25 @@ class QubitCircuit(object):
                                            gate.targets[0]))
             elif gate.name == "GLOBALPHASE":
                 self.U_list.append(globalphase(gate.arg_value, self.N))
+            elif gate.name in self.user_gates:
+                if gate.controls is not None:
+                    raise ValueError(
+                        "A user defined gate {} takes only  "
+                        "`targets` variable.".format(gate.name))
+                func = self.user_gates[gate.name]
+                para_num = len(inspect.getfullargspec(func)[0])
+                if para_num == 0:
+                    oper = func()
+                elif para_num == 1:
+                    oper = func(gate.arg_value)
+                else:
+                    raise ValueError(
+                        "gate function takes at most one parameters.")
+                self.U_list.append(expand_oper(oper, self.N, gate.targets))
+
+            else:
+                raise NotImplementedError(
+                    "{} gate is an unknown gate.".format(gate.name))
 
         return self.U_list
 
@@ -929,9 +1047,13 @@ class QubitCircuit(object):
                 if gate.targets and n in gate.targets:
 
                     if len(gate.targets) > 1:
-                        if ((self.reverse_states and n == max(gate.targets)) or
-                            (not self.reverse_states
-                             and n == min(gate.targets))):
+                        if gate.name == "SWAP":
+                            col.append(r" \qswap \qwx ")
+
+                        elif ((self.reverse_states and
+                                n == max(gate.targets)) or
+                                (not self.reverse_states and
+                                    n == min(gate.targets))):
                             col.append(r" \multigate{%d}{%s} " %
                                        (len(gate.targets) - 1,
                                         _gate_label(gate.name,
@@ -943,8 +1065,8 @@ class QubitCircuit(object):
 
                     elif gate.name == "CNOT":
                         col.append(r" \targ ")
-                    elif gate.name == "SWAP":
-                        col.append(r" \qswap ")
+                    elif gate.name == "TOFFOLI":
+                        col.append(r" \targ ")
                     else:
                         col.append(r" \gate{%s} " %
                                    _gate_label(gate.name, gate.arg_label))
@@ -952,15 +1074,12 @@ class QubitCircuit(object):
                 elif gate.controls and n in gate.controls:
                     m = (gate.targets[0] - n) * (-1 if self.reverse_states
                                                  else 1)
-                    if gate.name == "SWAP":
-                        col.append(r" \qswap \ctrl{%d} " % m)
-                    else:
-                        col.append(r" \ctrl{%d} " % m)
+                    col.append(r" \ctrl{%d} " % m)
 
                 elif (not gate.controls and not gate.targets):
                     # global gate
-                    if ((self.reverse_states and n == self.N - 1)
-                            or (not self.reverse_states and n == 0)):
+                    if ((self.reverse_states and n == self.N - 1) or
+                            (not self.reverse_states and n == 0)):
                         col.append(r" \multigate{%d}{%s} " %
                                    (self.N - 1,
                                     _gate_label(gate.name, gate.arg_label)))
@@ -974,10 +1093,14 @@ class QubitCircuit(object):
             col.append(r" \qw ")
             rows.append(col)
 
+        input_states = ["\lstick{\ket{" + x + "}}" if x is not None
+                        else "" for x in self.input_states]
+
         code = ""
         n_iter = (reversed(range(self.N)) if self.reverse_states
                   else range(self.N))
         for n in n_iter:
+            code += r" & %s" % input_states[n]
             for m in range(len(gates)):
                 code += r" & %s" % rows[m][n]
             code += r" & \qw \\ " + "\n"
