@@ -360,6 +360,13 @@ class SpinChain(CircuitProcessor):
 
         return qc_t
 
+    def optimize_circuit(self, qc):
+        self.qc0 = qc
+        self.qc1 = self.adjacent_gates(self.qc0)
+        self.qc2 = self.qc1.resolve_gates(
+            basis=["SQRTISWAP", "ISWAP", "RX", "RZ"])
+        return self.qc2
+
 
 class LinearSpinChain(SpinChain):
     """
@@ -380,11 +387,8 @@ class LinearSpinChain(SpinChain):
                 [r"$\sigma_x^%d\sigma_x^{%d} + \sigma_y^%d\sigma_y^{%d}$"
                  % (n, n, n + 1, n + 1) for n in range(self.N - 1)])
 
-    def optimize_circuit(self, qc):
-        self.qc0 = qc
-        self.qc1 = self.adjacent_gates(self.qc0, "linear")
-        self.qc2 = self.qc1.resolve_gates(basis=["ISWAP", "RX", "RZ"])
-        return self.qc2
+    def adjacent_gates(self, qc):
+        return super(LinearSpinChain, self).adjacent_gates(qc, "linear")
 
 
 class CircularSpinChain(SpinChain):
@@ -420,8 +424,5 @@ class CircularSpinChain(SpinChain):
                  % (n, n, (n + 1) % self.N, (n + 1) % self.N)
                  for n in range(self.N)])
 
-    def optimize_circuit(self, qc):
-        self.qc0 = qc
-        self.qc1 = self.adjacent_gates(self.qc0, "circular")
-        self.qc2 = self.qc1.resolve_gates(basis=["ISWAP", "RX", "RZ"])
-        return self.qc2
+    def adjacent_gates(self, qc):
+        return super(CircularSpinChain, self).adjacent_gates(qc, "circular")
