@@ -39,9 +39,10 @@ from types import FunctionType, BuiltinFunctionType
 from functools import partial
 
 from qutip.cy.codegen import Codegen
-from qutip.solver import Options, config
+from qutip.solver import Options, config, solver_safe
 from qutip.qobj import Qobj
 from qutip.superoperator import spre, spost
+from qutip.interpolate import Cubic_Spline
 
 
 def rhs_clear():
@@ -62,7 +63,12 @@ def rhs_clear():
     config.colexpect = None  # Placeholder for TD col_expect function.
     config.string = None     # Holds string of variables to be passed to solver
     config.tdname = None     # Name of td .pyx file (used in parallel mc code)
-
+    if "sesolve" in solver_safe:
+        del solver_safe["sesolve"]
+    if "mesolve" in solver_safe:
+        del solver_safe["mesolve"]
+    if "mcsolve" in solver_safe:
+        del solver_safe["mcsolve"]
 
 def rhs_generate(H, c_ops, args={}, options=Options(), name=None,
                  cleanup=True):
@@ -111,6 +117,7 @@ def rhs_generate(H, c_ops, args={}, options=Options(), name=None,
     Linds = []
     Lptrs = []
     Lcoeff = []
+    Lobj = []
 
     # loop over all hamiltonian terms, convert to superoperator form and
     # add the data of sparse matrix represenation to
@@ -148,6 +155,8 @@ def rhs_generate(H, c_ops, args={}, options=Options(), name=None,
             Ldata.append(L.data.data)
             Linds.append(L.data.indices)
             Lptrs.append(L.data.indptr)
+            if isinstance(h_coeff, Cubic_Spline):
+                Lobj.append(h_coeff.coeffs)
             Lcoeff.append(h_coeff)
 
         else:
@@ -230,6 +239,7 @@ def _td_format_check(H, c_ops, solver='me'):
     h_const = []
     h_func = []
     h_str = []
+    h_obj = []
     # check H for incorrect format
     if isinstance(H, Qobj):
         pass
@@ -248,6 +258,8 @@ def _td_format_check(H, c_ops, solver='me'):
                         h_func.append(k)
                     elif isinstance(H_k[1], str):
                         h_str.append(k)
+                    elif isinstance(H_k[1], Cubic_Spline):
+                        h_obj.append(k)
                     elif isinstance(H_k[1], np.ndarray):
                         h_str.append(k)
                     else:
@@ -259,22 +271,27 @@ def _td_format_check(H, c_ops, solver='me'):
     c_const = []
     c_func = []
     c_str = []
+    c_obj = []
     if isinstance(c_ops, list):
         for k in range(len(c_ops)):
             if isinstance(c_ops[k], Qobj):
                 c_const.append(k)
             elif isinstance(c_ops[k], list):
-                if len(c_ops[k]) != 2 or not isinstance(c_ops[k][0], Qobj):
+                if len(c_ops[k]) != 2:
                     raise TypeError(
-                        "Incorrect collapse operator specification")
+                        "Incorrect collapse operator specification.")
                 else:
                     if isinstance(c_ops[k][1], (FunctionType,
                                                 BuiltinFunctionType, partial)):
                         c_func.append(k)
                     elif isinstance(c_ops[k][1], str):
                         c_str.append(k)
+                    elif isinstance(c_ops[k][1], Cubic_Spline):
+                        c_obj.append(k)
                     elif isinstance(c_ops[k][1], np.ndarray):
                         c_str.append(k)
+                    elif isinstance(c_ops[k][1], tuple):
+                        c_str.append(k)
                     else:
                         raise TypeError(
                             "Incorrect collapse operator specification")
@@ -298,14 +315,32 @@ def _td_format_check(H, c_ops, solver='me'):
             raise Exception(
                 "Unable to load Cython. Use Python function format.")
         else:
-            if Cython.__version__ < '0.14':
+            if Cython.__version__ < '0.21':
                 raise Exception("Cython version (%s) is too old. Upgrade to " +
-                                "version 0.14+" % Cython.__version__)
+                                "version 0.21+" % Cython.__version__)
+
+    # If only time-dependence is in Objects, then prefer string based format
+    if (len(h_func) + len(c_func) + len(h_str) + len(c_str)) == 0:
+         h_str += h_obj #Does nothing if not objects
+         c_str += c_obj
+    else:
+        # Combine Hamiltonian objects
+        if len(h_func) > 0:
+            h_func += h_obj
+        elif len(h_str) > 0:
+            h_str += h_obj
+
+        #Combine collapse objects
+        if len(c_func) > 0:
+            c_func += c_obj
+        elif len(c_str) > 0:
+            c_str += c_obj
 
     if solver == 'me':
         return (len(h_const + c_const),
                 len(h_func) + len(c_func),
                 len(h_str) + len(c_str))
+
     elif solver == 'mc':
 
         #   H      C_ops    #
@@ -375,7 +410,7 @@ def _td_wrap_array_str(H, c_ops, args, times):
             if isinstance(Hk, list) and isinstance(Hk[1], np.ndarray):
                 H_op, H_td = Hk
                 td_array_name = "_td_array_%d" % n
-                H_td_str = '(0 if (t > %f) else %s[round(%d * (t/%f))])' %\
+                H_td_str = '(0 if (t > %f) else %s[int(round(%d * (t/%f)))])' %\
                     (times[-1], td_array_name, len(times) - 1, times[-1])
                 args_new[td_array_name] = H_td
                 H_new.append([H_op, H_td_str])
@@ -390,7 +425,7 @@ def _td_wrap_array_str(H, c_ops, args, times):
             if isinstance(ck, list) and isinstance(ck[1], np.ndarray):
                 c_op, c_td = ck
                 td_array_name = "_td_array_%d" % n
-                c_td_str = '(0 if (t > %f) else %s[round(%d * (t/%f))])' %\
+                c_td_str = '(0 if (t > %f) else %s[int(round(%d * (t/%f)))])' %\
                     (times[-1], td_array_name, len(times) - 1, times[-1])
                 args_new[td_array_name] = c_td
                 c_ops_new.append([c_op, c_td_str])
