@@ -47,11 +47,14 @@ import scipy.linalg as la
 from scipy.linalg.blas import get_blas_funcs
 _dznrm2 = get_blas_funcs("znrm2")
 from qutip.cy.sparse_utils import (_sparse_profile, _sparse_permute,
-                                   _sparse_reverse_permute, _sparse_bandwidth)
+                                   _sparse_reverse_permute, _sparse_bandwidth,
+                                   _isdiag, zcsr_one_norm, zcsr_inf_norm)
+from qutip.fastsparse import fast_csr_matrix
+from qutip.cy.spconvert import (arr_coo2fast, zcsr_reshape)
 from qutip.settings import debug
 
-import qutip.logging
-logger = qutip.logging.get_logger()
+import qutip.logging_utils
+logger = qutip.logging_utils.get_logger()
 
 if debug:
     import inspect
@@ -65,40 +68,40 @@ def sp_fro_norm(data):
     return np.sqrt(out)
 
 
-def sp_inf_norm(data):
+def sp_inf_norm(A):
     """
     Infinity norm for sparse matrix
     """
-    return np.max([np.sum(np.abs(data.getrow(k).data))
-                   for k in range(data.shape[0])])
+    return zcsr_inf_norm(A.data, A.indices, 
+                A.indptr, A.shape[0], A.shape[1])
 
 
-def sp_L2_norm(data):
+def sp_L2_norm(A):
     """
     L2 norm sparse vector
     """
-    if 1 not in data.shape:
+    if 1 not in A.shape:
         raise TypeError("Use L2-norm only for vectors.")
 
-    if len(data.data):
-        return _dznrm2(data.data)
+    if len(A.data):
+        return _dznrm2(A.data)
     else:
         return 0
 
 
-def sp_max_norm(data):
+def sp_max_norm(A):
     """
     Max norm for sparse matrix
     """
-    return np.max(np.abs(data.data)) if any(data.data) else 0
+    return np.max(np.abs(A.data)) if any(A.data) else 0
 
 
-def sp_one_norm(data):
+def sp_one_norm(A):
     """
     One norm for sparse matrix
     """
-    return np.max(np.array([np.sum(np.abs((data.getcol(k).data)))
-                            for k in range(data.shape[1])]))
+    return zcsr_one_norm(A.data, A.indices, 
+                A.indptr, A.shape[0], A.shape[1])
 
 
 def sp_reshape(A, shape, format='csr'):
@@ -127,7 +130,10 @@ def sp_reshape(A, shape, format='csr'):
     """
     if not hasattr(shape, '__len__') or len(shape) != 2:
         raise ValueError('Shape must be a list of two integers')
-
+    
+    if format == 'csr':
+        return zcsr_reshape(A, shape[0], shape[1])
+    
     C = A.tocoo()
     nrows, ncols = C.shape
     size = nrows * ncols
@@ -138,11 +144,9 @@ def sp_reshape(A, shape, format='csr'):
 
     flat_indices = ncols * C.row + C.col
     new_row, new_col = divmod(flat_indices, shape[1])
-    B = sp.coo_matrix((C.data, (new_row, new_col)), shape=shape)
 
-    if format == 'csr':
-        return B.tocsr()
-    elif format == 'coo':
+    B = sp.coo_matrix((C.data, (new_row, new_col)), shape=shape)
+    if format == 'coo':
         return B
     elif format == 'csc':
         return B.tocsc()
@@ -377,96 +381,20 @@ def sp_eigs(data, isherm, vecs=True, sparse=False, sort='low',
     return (evals, evecs) if vecs else evals
 
 
-def sp_expm(data, sparse=True):
+def sp_expm(A, sparse=False):
     """
-    Sparse matrix exponential.
+    Sparse matrix exponential.    
     """
-    A = data.tocsc()  # extract Qobj data (sparse matrix)
-    m_vals = np.array([3, 5, 7, 9, 13])
-    theta = np.array([0.01495585217958292, 0.2539398330063230,
-                      0.9504178996162932, 2.097847961257068,
-                      5.371920351148152], dtype=float)
-    normA = sp_one_norm(data)
-    if normA <= theta[-1]:
-        for ii in range(len(m_vals)):
-            if normA <= theta[ii]:
-                F = _pade(A, m_vals[ii], sparse)
-                break
+    if _isdiag(A.indices, A.indptr, A.shape[0]):
+        A = sp.diags(np.exp(A.diagonal()), shape=A.shape, 
+                    format='csr', dtype=complex)
+        return A
+    if sparse:
+        E = spla.expm(A.tocsc())
     else:
-        t, s = np.frexp(normA / theta[-1])
-        s = s - (t == 0.5)
-        A = A / 2.0 ** s
-        F = _pade(A, m_vals[-1], sparse)
-        for i in range(s):
-            F = F * F
-
-    return F
-
-
-def _pade(A, m, sparse):
-    n = np.shape(A)[0]
-    c = _padecoeff(m)
-
-    if m != 13:
-        apows = [[] for jj in range(int(np.ceil((m + 1) / 2)))]
-        apows[0] = sp.eye(n, n, format='csc')
-        apows[1] = A * A
-        for jj in range(2, int(np.ceil((m + 1) / 2))):
-            apows[jj] = apows[jj - 1] * apows[1]
-        U = sp.lil_matrix((n, n)).tocsc()
-        V = sp.lil_matrix((n, n)).tocsc()
-        for jj in range(m, 0, -2):
-            U = U + c[jj] * apows[jj // 2]
-        U = A * U
-        for jj in range(m - 1, -1, -2):
-            V = V + c[jj] * apows[(jj + 1) // 2]
-
-        if sparse:
-            F = spla.spsolve((-U + V), (U + V))
-            return F.tocsr()
-        else:
-            F = la.solve((-U + V).todense(), (U + V).todense())
-            return sp.lil_matrix(F).tocsr()
-
-    elif m == 13:
-        A2 = A * A
-        A4 = A2 * A2
-        A6 = A2 * A4
-        U = A * (A6 * (c[13] * A6 + c[11] * A4 + c[9] * A2) +
-                 c[7] * A6 + c[5] * A4 + c[3] * A2 +
-                 c[1] * sp.eye(n, n).tocsc())
-        V = A6 * (c[12] * A6 + c[10] * A4 + c[8] * A2) + c[6] * A6 + c[4] * \
-            A4 + c[2] * A2 + c[0] * sp.eye(n, n).tocsc()
-
-        if sparse:
-            F = spla.spsolve((-U + V), (U + V))
-            return F.tocsr()
-        else:
-            F = la.solve((-U + V).todense(), (U + V).todense())
-            return sp.csr_matrix(F)
-
-
-def _padecoeff(m):
-    """
-    Private function returning coefficients for Pade approximation.
-    """
-    if m == 3:
-        return np.array([120, 60, 12, 1])
-    elif m == 5:
-        return np.array([30240, 15120, 3360, 420, 30, 1])
-    elif m == 7:
-        return np.array([17297280, 8648640, 1995840, 277200,
-                         25200, 1512, 56, 1])
-    elif m == 9:
-        return np.array([17643225600, 8821612800, 2075673600,
-                         302702400, 30270240, 2162160, 110880,
-                         3960, 90, 1])
-    elif m == 13:
-        return np.array([64764752532480000, 32382376266240000,
-                         7771770303897600, 1187353796428800,
-                         129060195264000, 10559470521600, 670442572800,
-                         33522128640, 1323241920, 40840800,
-                         960960, 16380, 182, 1])
+        E = spla.expm(A.toarray())
+    return sp.csr_matrix(E)
+    
 
 
 def sp_permute(A, rperm=(), cperm=(), safe=True):
@@ -519,7 +447,7 @@ def sp_permute(A, rperm=(), cperm=(), safe=True):
     data, ind, ptr = _sparse_permute(A.data, A.indices, A.indptr,
                                      nrows, ncols, rperm, cperm, flag)
     if kind == 'csr':
-        return sp.csr_matrix((data, ind, ptr), shape=shp, dtype=data.dtype)
+        return fast_csr_matrix((data, ind, ptr), shape=shp)
     elif kind == 'csc':
         return sp.csc_matrix((data, ind, ptr), shape=shp, dtype=data.dtype)
 
@@ -575,7 +503,7 @@ def sp_reverse_permute(A, rperm=(), cperm=(), safe=True):
                                              nrows, ncols, rperm, cperm, flag)
 
     if kind == 'csr':
-        return sp.csr_matrix((data, ind, ptr), shape=shp, dtype=data.dtype)
+        return fast_csr_matrix((data, ind, ptr), shape=shp)
     elif kind == 'csc':
         return sp.csc_matrix((data, ind, ptr), shape=shp, dtype=data.dtype)
 
@@ -642,3 +570,22 @@ def sp_profile(A):
         raise TypeError('Input sparse matrix must be in CSR or CSC format.')
 
     return up+lp, lp, up
+
+
+def sp_isdiag(A):
+    """Determine if sparse CSR matrix is diagonal.
+    
+    Parameters
+    ----------
+    A : csr_matrix, csc_matrix
+        Input matrix
+        
+    Returns
+    -------
+    isdiag : int
+        True if matix is diagonal, False otherwise.
+    
+    """
+    if not sp.isspmatrix_csr(A):
+        raise TypeError('Input sparse matrix must be in CSR format.')
+    return _isdiag(A.indices, A.indptr, A.shape[0])
