@@ -32,15 +32,20 @@
 ###############################################################################
 
 from __future__ import division
-from qutip import *
-from numpy import allclose, linspace, log2
+
+import numpy as np
 from numpy.testing import assert_, assert_equal, run_module_suite
 
+from qutip import (basis, ket2dm, cnot, entropy_vn, entropy_linear, rand_ket,
+                   rand_dm, tensor, concurrence, entropy_mutual, ptrace,
+                   entropy_conditional, entangling_power, iswap, swap,
+                   berkeley, sqrtswap, swapalpha)
+
 
 def test_EntropyVN():
-    "von-Neumann entropy"
+    "Entropy: von-Neumann entropy"
     # verify that entropy_vn gives correct binary entropy
-    a = linspace(0, 1, 20)
+    a = np.linspace(0, 1, 20)
     for k in range(len(a)):
         # a*|0><0|
         x = a[k] * ket2dm(basis(2, 0))
@@ -52,8 +57,8 @@ def test_EntropyVN():
         if k == 0 or k == 19:
             assert_equal(out, -0.0)
         else:
-            assert_(abs(-out - a[k] * log2(a[k])
-                        - (1. - a[k]) * log2((1. - a[k]))) < 1e-12)
+            assert_(abs(-out - a[k] * np.log2(a[k])
+                        - (1. - a[k]) * np.log2((1. - a[k]))) < 1e-12)
 
     # test_ entropy_vn = 0 for pure state
     psi = rand_ket(10)
@@ -61,7 +66,7 @@ def test_EntropyVN():
 
 
 def test_EntropyLinear():
-    "Linear entropy"
+    "Entropy: Linear entropy"
     # test_ entropy_vn = 0 for pure state
     psi = rand_ket(10)
     assert_equal(abs(entropy_linear(psi)) <= 1e-13, True)
@@ -73,7 +78,7 @@ def test_EntropyLinear():
 
 
 def test_EntropyConcurrence():
-    "Concurrence"
+    "Entropy: Concurrence"
     # check concurrence = 1 for maximal entangled (Bell) state
     bell = ket2dm(
         (tensor(basis(2), basis(2)) + tensor(basis(2, 1), basis(2, 1))).unit())
@@ -86,7 +91,7 @@ def test_EntropyConcurrence():
 
 
 def test_EntropyMutual():
-    "Mutual information"
+    "Entropy: Mutual information"
     # verify mutual information = S(A)+S(B) for pure state
     rhos = [rand_dm(25, dims=[[5, 5], [5, 5]], pure=True) for k in range(10)]
 
@@ -105,7 +110,7 @@ def test_EntropyMutual():
 
 
 def test_EntropyConditional():
-    "Conditional entropy"
+    "Entropy: Conditional entropy"
     # test_ S(A,B|C,D)<=S(A|C)+S(B|D)
     rhos = [rand_dm(16, dims=[[2, 2, 2, 2], [2, 2, 2, 2]], pure=True)
             for k in range(20)]
@@ -127,14 +132,14 @@ def test_EntropyConditional():
 
 
 def test_EntanglingPower():
-    "Entangling power"
+    "Entropy: Entangling power"
     assert_(abs(entangling_power(cnot()) - 2/9) < 1e-12)
     assert_(abs(entangling_power(iswap()) - 2/9) < 1e-12)
     assert_(abs(entangling_power(berkeley()) - 2/9) < 1e-12)
     assert_(abs(entangling_power(sqrtswap()) - 1/6) < 1e-12)
-    alpha = 2 * pi * np.random.rand()
+    alpha = 2 * np.pi * np.random.rand()
     assert_(abs(entangling_power(swapalpha(alpha))
-                - 1/6 * sin(pi * alpha) ** 2) < 1e-12)
+                - 1/6 * np.sin(np.pi * alpha) ** 2) < 1e-12)
     assert_(abs(entangling_power(swap()) - 0) < 1e-12)
 
 
