@@ -42,10 +42,9 @@ pwd = os.path.dirname(__file__)
 from qutip import (rand_dm, graph_degree, breadth_first_search,
                    reverse_cuthill_mckee, tensor, destroy, qeye, sigmam,
                    liouvillian, maximum_bipartite_matching,
-                   weighted_bipartite_matching)
+                   weighted_bipartite_matching, column_permutation)
 from qutip.sparse import sp_permute, sp_bandwidth
 
-
 def test_graph_degree():
     "Graph: Graph Degree"
     A = rand_dm(25, 0.1)
@@ -90,15 +89,6 @@ def test_graph_rcm_simple():
     assert_equal((perm - ans).all(), 0)
 
 
-def test_graph_rcm_bucky():
-    "Graph: Reverse Cuthill-McKee Ordering (Bucky)"
-    B = np.load(pwd+'/bucky.npy')
-    B = sp.csr_matrix(B, dtype=float)
-    perm = reverse_cuthill_mckee(B)
-    ans = np.load(pwd+'/bucky_perm.npy')
-    assert_equal(perm, ans)
-
-
 def test_graph_rcm_boost():
     "Graph: Reverse Cuthill-McKee Ordering (boost)"
     M = np.zeros((10, 10))
@@ -110,7 +100,7 @@ def test_graph_rcm_boost():
     M[5, [6, 7]] = 1
     M[6, 7] = 1
     M = M+M.T
-    M = sp.csr_matrix(M)
+    M = sp.csr_matrix(M, dtype=complex)
     perm = reverse_cuthill_mckee(M, 1)
     ans_perm = np.array([9, 7, 6, 4, 1, 5, 0, 2, 3, 8])
     assert_equal((perm - ans_perm).all(), 0)
@@ -165,5 +155,13 @@ def test_graph_weighted_matching():
     assert_equal(np.sum(A.diagonal()), np.sum(C.diagonal()))
 
 
+def test_column_permutation():
+    "Graph: Column Permutation"
+    A = sp.rand(5, 5, 0.25, format='csc')
+    perm = column_permutation(A)
+    B = sp_permute(A, [], perm)
+    counts = np.diff(B.indptr)
+    assert_equal(np.all(np.argsort(counts) == np.arange(5)), True)
+
 if __name__ == "__main__":
     run_module_suite()
