@@ -34,7 +34,8 @@
 import numpy as np
 from numpy.testing import assert_equal, run_module_suite
 
-from qutip import (jmat, basis, destroy, create, displace, qeye, num, squeeze)
+from qutip import (jmat, basis, destroy, create, displace, qeye, 
+                    num, squeeze, charge, tunneling)
 
 
 def test_jmat_12():
@@ -42,7 +43,7 @@ def test_jmat_12():
     spinhalf = jmat(1 / 2.)
 
     paulix = np.array([[0.0 + 0.j, 0.5 + 0.j], [0.5 + 0.j, 0.0 + 0.j]])
-    pauliy = np.array([[0. + 0.j, 0. + 0.5j], [0. - 0.5j, 0. + 0.j]])
+    pauliy = np.array([[0. + 0.j, 0. - 0.5j], [0. + 0.5j, 0. + 0.j]])
     pauliz = np.array([[0.5 + 0.j, 0.0 + 0.j], [0.0 + 0.j, -0.5 + 0.j]])
     sigmap = np.array([[0. + 0.j, 1. + 0.j], [0. + 0.j, 0. + 0.j]])
     sigmam = np.array([[0. + 0.j, 0. + 0.j], [1. + 0.j, 0. + 0.j]])
@@ -65,10 +66,10 @@ def test_jmat_32():
          [0.0000000 + 0.j, 0.0000000 + 0.j, 0.8660254 + 0.j, 0.0000000 + 0.j]])
 
     pauliy32 = np.array(
-        [[0. + 0.j, 0. + 0.8660254j, 0. + 0.j, 0. + 0.j],
-         [0. - 0.8660254j, 0. + 0.j, 0. + 1.j, 0. + 0.j],
-         [0. + 0.j, 0. - 1.j, 0. + 0.j, 0. + 0.8660254j],
-         [0. + 0.j, 0. + 0.j, 0. - 0.8660254j, 0. + 0.j]])
+        [[0. + 0.j, 0. - 0.8660254j, 0. + 0.j, 0. + 0.j],
+         [0. + 0.8660254j, 0. + 0.j, 0. - 1.j, 0. + 0.j],
+         [0. + 0.j, 0. + 1.j, 0. + 0.j, 0. - 0.8660254j],
+         [0. + 0.j, 0. + 0.j, 0. + 0.8660254j, 0. + 0.j]])
 
     pauliz32 = np.array([[1.5 + 0.j, 0.0 + 0.j, 0.0 + 0.j, 0.0 + 0.j],
                          [0.0 + 0.j, 0.5 + 0.j, 0.0 + 0.j, 0.0 + 0.j],
@@ -89,7 +90,7 @@ def test_jmat_42():
 def test_jmat_52():
     "Spin 5/2 operators"
     spin52 = jmat(5 / 2., '+')
-    assert_equal(spin52.shape == [6, 6], True)
+    assert_equal(spin52.shape == (6, 6), True)
 
 
 def test_destroy():
@@ -129,7 +130,7 @@ def test_qeye():
 
 
 def test_qeye_dims():
-    "Identity operator"
+    "Identity operator (array input)"
     eye24 = qeye([2, 3, 4])
     assert_equal(np.allclose(eye24.full(), np.eye(24, dtype=complex)), True)
     assert_equal(eye24.dims, [[2, 3, 4], [2, 3, 4]])
@@ -175,5 +176,26 @@ def test_displace():
     assert_equal(np.allclose(dp.full(), dpmatrix), True)
 
 
+def test_charge():
+    "Charge operator"
+    N = 5
+    M = - np.random.randint(N)
+    ch = charge(N,M)
+    ch_matrix = np.diag(np.arange(M,N+1))
+    assert_equal(np.allclose(ch.full(), ch_matrix), True)
+
+
+def test_tunneling():
+    "Tunneling operator"
+    N = 5
+    tn = tunneling(2*N+1)
+    tn_matrix = np.diag(np.ones(2*N),k=-1) + np.diag(np.ones(2*N),k=1)
+    assert_equal(np.allclose(tn.full(), tn_matrix), True) 
+    
+    tn = tunneling(2*N+1,2)
+    tn_matrix = np.diag(np.ones(2*N-1),k=-2) + np.diag(np.ones(2*N-1),k=2)
+    assert_equal(np.allclose(tn.full(), tn_matrix), True)
+
+
 if __name__ == "__main__":
     run_module_suite()
