@@ -31,9 +31,12 @@
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
 
-from numpy.testing import assert_, run_module_suite
-from qutip.qip.gates import gate_sequence_product
+import numpy as np
+from numpy.testing import assert_, assert_allclose, run_module_suite
+from qutip.qip.gates import gate_sequence_product, rx, identity, expand_oper
 from qutip.qip.circuit import QubitCircuit, Gate
+from qutip.tensor import tensor
+from qutip.qobj import Qobj
 
 
 class TestQubitCircuit:
@@ -147,6 +150,9 @@ class TestQubitCircuit:
         test_gate = Gate("RZ", targets=[1], arg_value = 1.570796,
                          arg_label="P")
         qc.add_gate(test_gate)
+        qc.add_gate("TOFFOLI", controls=[0, 1], targets=[2])
+        qc.add_gate("SNOT", targets=[3])
+        qc.add_gate(test_gate, index = [3])
 
         # Test explicit gate addition
         assert_(qc.gates[0].name == "CNOT")
@@ -158,6 +164,11 @@ class TestQubitCircuit:
         assert_(qc.gates[1].targets == test_gate.targets)
         assert_(qc.gates[1].controls == test_gate.controls)
 
+        # Test specified position gate addition
+        assert_(qc.gates[3].name == test_gate.name)
+        assert_(qc.gates[3].targets == test_gate.targets)
+        assert_(qc.gates[3].controls == test_gate.controls)
+
     def test_add_state(self):
         """
         Addition of input and output states to a circuit.
@@ -220,5 +231,32 @@ class TestQubitCircuit:
         assert_(qc.input_states[2] == None)
         assert_(qc.output_states[1] == "+")
 
+    def test_user_gate(self):
+        """
+        User defined gate for QubitCircuit
+        """
+        def customer_gate1(arg_values):
+            mat = np.zeros((4, 4), dtype=np.complex)
+            mat[0, 0] = mat[1, 1] = 1.
+            mat[2:4, 2:4] = rx(arg_values)
+            return Qobj(mat, dims=[[2, 2], [2, 2]])
+
+        def customer_gate2(arg_values):
+            mat = np.array([[1.,   0],
+                            [0., 1.j]])
+            return Qobj(mat, dims=[[2], [2]])
+
+        qc = QubitCircuit(3)
+        qc.user_gates = {"CTRLRX": customer_gate1,
+                         "T": customer_gate2}
+        qc.add_gate("CTRLRX", targets=[1, 2], arg_value=np.pi/2)
+        qc.add_gate("T", targets=[1])
+        props = qc.propagators()
+        result1 = tensor(identity(2), customer_gate1(np.pi/2))
+        assert_allclose(props[0], result1)
+        result2 = tensor(identity(2), customer_gate2(None), identity(2))
+        assert_allclose(props[1], result2)
+
+
 if __name__ == "__main__":
     run_module_suite()
