@@ -64,8 +64,10 @@ Contributed by Markus Baden, Oct. 07, 2011
 
 '''
 
+__all__ = ['three_level_basis', 'three_level_ops']
+
 from qutip.states import qutrit_basis
-from scipy import array
+from numpy import array
 
 
 def three_level_basis():
@@ -101,4 +103,4 @@ def three_level_ops():
     sig33 = three * three.dag()
     sig12 = one * two.dag()
     sig32 = three * two.dag()
-    return array([sig11, sig22, sig33, sig12, sig32])
+    return array([sig11, sig22, sig33, sig12, sig32], dtype=object)
