@@ -31,16 +31,18 @@
 #    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
 
+__all__ = ['qpt_plot', 'qpt_plot_combined', 'qpt']
+
 from qutip.tensor import tensor
 from qutip.superoperator import spre, spost, mat2vec, vec2mat
-from qutip.qobj import Qobj
 from numpy import hstack, real, imag
 import scipy.linalg as la
 from qutip.visualization import matrix_histogram, matrix_histogram_complex
-import qutip.settings
 
-if qutip.settings.qutip_graphics == 'YES':
+try:
     import matplotlib.pyplot as plt
+except:
+    pass
 
 
 def _index_permutations(size_list, perm=[]):
