@@ -141,8 +141,8 @@ def clebsch(j1, j2, j3, m1, m2, m3):
         Requested Clebsch-Gordan coefficient.
 
     """
-    from scipy.misc import factorial
-    
+    from scipy.special import factorial
+
     if m3 != m1 + m2:
         return 0
     vmin = int(np.max([-j1 + j2 + m3, -j1 + m1, 0]))
@@ -413,4 +413,4 @@ def _blas_info():
         blas = 'Accelerate'
     else:
         blas = 'Generic'
-    return blas
\ No newline at end of file
+    return blas
